#pragma once
#include "View.h"
#include "Graph.h"
#include <map>

/*!
 * This class is a Containerclass for creating the ballon tree. It provides function for setting and getting information for creating the tree, like the radius or the confidence of the tree(part).
 * @author Michael Beham
 * @date 4.6.2011
 */
class VertexBallon
{
public:
	typedef boost::shared_ptr<VertexBallon> VertexBallonPtr;

	/*!
	 * Standard constructor
	 */
	VertexBallon(); 
	/*!
	 * constructor which creates a object corresponding the passed vertex.
	 * @param v [VertexPtr] vertex node
	 */
	VertexBallon(VertexPtr v); 
	/*!
	 * Constructor whichs creates a object corresponding the passed Vertex and states.
	 * @param v [VertexPtr] node
	 * @param r [double] radius
	 * @param rr [double] rendering radius
	 * @param circumference [double] circumference
	 * @param ark [double] ark
	 * @param angle [double] angle
	 */
	VertexBallon(VertexPtr v, double r, double rr, double circumference, double ark, double angle); 
	
	/*!
	 * Returns the corresponding Vertex
	 * @return [VertexPtr] corresponding node
	 */
	VertexPtr GetVertex();
	/*! 
	 * Returns the corresponding radius
	 * @return [double] radius
	 */
	double GetRadius();
	/*! 
	 * Returns the corresponding radius
	 * @return [double] radius for rendering
	 */
	double GetRadiusRendering();
	/*! 
	 * Returns the corresponding ark
	 * @return [double] ark
	 */
	double GetArk();
	/*! 
	 * Returns the corresponding angle
	 * @return [double] angle
	 */
	double GetAngle();
	/*! 
	 * Returns the corresponding circumference
	 * @return [double] circumference
	 */
	double GetCircumference();

	/*! 
	 * Sets the corresponding tree node
	 * @param v [VertexPtr] corresponding tree node
	 */
	void SetVertex(VertexPtr v);
	/*! 
	 * Sets the radius
	 * @param v [VertexPtr] radius
	 */
	void SetRadius(double r);
	/*! 
	 * Sets the radius for rendering
	 * @param v [VertexPtr] radius for rendering
	 */
	void SetRadiusRendering(double r);
	/*! 
	 * Sets the ark
	 * @param v [VertexPtr] ark
	 */
	void SetArk(double r);
	/*! 
	 * Sets the angle
	 * @param v [VertexPtr] angle
	 */
	void SetAngle(double angle);
	/*! 
	 * Sets the circumference
	 * @param v [VertexPtr] circumference
	 */
	void SetCircumference(double circumference);

private:
	VertexPtr m_pkV;
	double m_r;
	double m_rr;
	double m_ark;
	double m_angle;
	double m_circumference;
};

/*! 
 * This class creates the ballon tree layout. It's child of View and the Calculate method creates the layout
 * @author Michael Beham
 * @date 4.6.2011
 */
class BallonTree :	public View
{
	typedef boost::shared_ptr<VertexBallon> VertexBallonPtr;

public:
	/*!
	 * Standard constructor
	 */
	BallonTree();
	/*!
	 * Standard deconstructor
	 */
	~BallonTree(void);

	/*!
	 * This method calculates the tre layout 
	 */
	virtual void Calculate();

	/*std::vector<VertexBallonPtr> verticesChilds;/*/
	std::map<unsigned int, VertexBallonPtr> verticesChilds;//*/
};
