#pragma once

#include <stdlib.h>
#include <iostream>


class BasisFunktion
{

public:
	/*! Constructor of a new BasisFunktion Object
	* @param numberControlPoints number of Controlpoints for the B-Spline
	* @param splinePoints number of Points in the line-strip
	*/
	BasisFunktion(int numberControlPoints, int splinePoints);
	/*! Destructor*/
	~BasisFunktion();
	/*! return the number of Points in the line-strip
	* @return number of Points in the line-strip 
	*/
	int getNumberSplinePoints();
	/*! return the Value according to the given Parameter 
	*@param i indicates the Basisfunktion
	*@param splinePoint Index of the Splinepoint
	*@return the weighted value stored at the given position
	*/
	double getValue(int i, int splinePoint);

private:
	/*! Number of control points for a B-Spline*/
	int numberControlPoints;
	/*! Number of points to draw a line-strip*/
	int splinePoints;
	/*! Degree of the Polynom, is equal to degree-1 */
	int p;
	/*! Length of the knot vector, must be numberControlPoints + degree of the curve + 1 */
	int n;
	/*! the knotvector*/
	int *knotVector;
	/*! The ValueArray is filled with the weights of the control point, to a splinePoint */
	double **valueArray;

	/*! initializes a uniform knot vector*/
	void initKnotVector();
	/*! initialize and fill the value array with the calculated weights*/
	void initValueArray();
	/*! returns the calculated weight of the control Points for the B-Spline using the recursion formula of Cox... 
	* @param i index from 1 to n-p
	* @param p degree of the curve
	* @param u parameter where the function is to be evaluated
	* @return the calculated weighting value
	*/
	double coxRecursion(int i, int p, double u);

};