#pragma once
#include "Vertex.h"
#include <memory> 

#include <boost/ptr_container/ptr_deque.hpp>
#include <boost/ptr_container/ptr_map.hpp>
#include <boost/assign/ptr_map_inserter.hpp>  // for 'ptr_map_insert()'

using namespace boost;
using namespace boost::assign;

/*!
 * This class specififes an edge.
 * @author Michael Beham
 * @date 3.6.2011
 */
class Edge 
{
public:
	typedef boost::shared_ptr<Vertex> VertexPtr;
	/*!
	 * Standardconstructor
	 * @param [VertexPtr] start startvertex
	 * @param [VertexPtr] end endvertex.
	 */
	Edge(VertexPtr start, VertexPtr end);
	/*!
	 * Standarddeconstructor
	 */
	virtual ~Edge(void);

public:
	/*! 
	 * This method sets the start and end vertex of the edge
	 * @param [VertexPtr] start start vertex
	 * @param [VertexPtr] end end vertex
	 */
	void SetEdges(VertexPtr start, VertexPtr end);
	/*!
	 * This method sets the start edge.
	 * @param [VertexPtr] start start edge.
	 */
	void SetStartEdge(VertexPtr start);
	/*!
	 * This method sets the end edge.
	 * @param [VertexPtr] end end edge.
	 */
	void SetEndEdge(VertexPtr end);
	/*!
	 * This method returns the start edge
	 * @return [VertexPtr] startedge
	 */
	VertexPtr GetStartEdge();
	/*!
	 * This method returns the end edge
	 * @return [VertexPtr] endedge
	 */
	VertexPtr GetEndEdge();

	/*! 
	 * This method sets the ID of the edge
	 * @param id [unsigned int] id of the edge
	 */
	void SetID(unsigned int id);
	/*! 
	 * This method returns the ID of the edge
	 * @return [unsigned int] id of the edge
	 */
	unsigned int GetID();

protected:
	VertexPtr m_start;
	VertexPtr m_end;

	unsigned int m_id;
};
