#pragma once
#include "glm/glm.hpp"

using glm::vec3;
using glm::vec4;
using glm::mat4;

/*!
 * This class saves the coordinate, color, and alpha for a GUIVertex. 
 * @author Michael Beham
 * @date 6.6.2010
 */
class GUIVertex
{
public:
	/*!
	 * standard constructor
	 */
	GUIVertex();
	/*!
	 * constructor to save the coordinates
	 * @param position [vec3] coordinates
	 */
	GUIVertex(vec3 position);
	/*!
	 * constructor to save the coordinates and color
	 * @param position [vec3] coordinates
	 * @param color [vec3] color
	 */
	GUIVertex(vec3 position, vec3 color);
	/*!
	 * constructor to save the coordinates, color, and alpha value
	 * @param position [vec3] coordinates
	 * @param color [vec3] color
	 * @param color [vec3] alpha
	 */
	GUIVertex(vec3 position, vec3 color, float opacity);
	/*!
	 * standard deconstructor
	 */
	~GUIVertex(void);

	/*!
	 * This method sets the coordinate
	 * @param pos [vec3] coordinate
	 */
	void SetPosition(vec3 pos);
	/*!
	 * This method returns the coordinate
	 * @return [vec3] coordinate
	 */
	vec3 GetPosition();

	/*!
	 * This method sets the coordinate
	 * @param pos [vec3] coordinate
	 */
	void SetColor(vec3 color);
	/*!
	 * This method returns the color
	 * @return[vec3] color
	 */
	vec3 GetColor();

	/*!
	 * This method sets the alpha value
	 * @param opacity [float] opacity
	 */
	void SetOpacity(float opacity);
	/*!
	 * This method returns the alpha value
	 * @return[float] opacity
	 */
	float GetOpacity();
private:
	vec3 m_position;
	vec4 m_color;
};

