#pragma once
#include <IL/ilu.h>
#include <IL/il.h>
#include <iostream>
#include <vector>

/*!
 * This class provides method to load images.
 * It use the DevIL package to read the images.
 * @author Michael Beham
 * @date 3.6.2011
 */
class Image
{
	public:
		/*!
		 * Standardconstructor
		 */
		Image():imageload(false){};
		/*!
		 * Standarddeconstructor
		 */
		~Image(){ilDeleteImages(1, & handle);free(data); data = NULL;};
		/*!
		 * This method returns true, if a images is loaded, or false, if no image is loaded.
		 * @return [bool] is an image loaded?
		 */
		bool IsImageLoad();
		/*!
		 * This method reads a image. 
		 * @param Type [ILenum] file extension of the type
		 * @param FileName [char *] path to the image
		 * @return [bool] true... if the loading was succesfull.
		 */
		bool LoadImg(ILenum Type, char *FileName);

		/*!
		 * This method reads a image and returns the data. 
		 * @param Type [ILenum] file extension of the type
		 * @param FileName [char *] path to the image
		 * @return [ILfloat] Imagedata.
		 */
		static ILfloat* LoadImgGetFloat(ILenum Type, char *FileName, int &width, int &height);

		/*!
		 * This method returns the width of the image
		 * @return [int] width of the image
		 */
		int GetWidth();
		/*!
		 * This method returns the height of the image
		 * @return [int] height of the image
		 */
		int GetHeight();
		/*!
		 * This method returns the data of the image
		 * @return [ILubyte*] data of the image
		 */
		ILubyte* GetData();
		ILuint handle;

		static std::vector<Image*> images;

		double id;
	private:
		bool imageload;
		ILuint width, height;		
		ILubyte * data;
};
