#pragma once

#include <tinyxml.h>
#include <string>
#include <vector>
#include <iostream>

using namespace std;

//!  LoadData class. 
/*!
 * This class provides functions for saving and loading a XML file. 
 * @author Michael Beham 
*/

class LoadData
{
public:
	/**
	* constructor: 
	**/
	LoadData();
	/*!
	 * dekonstruktor
	 */
	virtual ~LoadData(){};
	
	/*!
	 * This method convert a string to a char*
	 * @param string [std::string] input string
	 * @return [char*] output character array
	 */
	static char* ConvertStringToChar(std::string string);
	/*!
	 * This method converts a integer to string
	 * @param i [int] integervalue
	 * @return [string] stringvalue
	 */
	std::string GetIntegerToString(int i);

protected:
	/*!
	 * This method reads a XML file
	 * @param file [const char*] path to XML file
	 */
	void ReadXMLFile(const char* file);
	/*!
	 * This method reads a XML file
	 * @param [const char*] path to XML file
	 */
	LoadData(const char* file);	
	/*!
	 * This method loads the attributes
	 * @param pElement [TiXMLElement*] the XML element
	 * @param indent [int] indent
	 */
	virtual int attribsLoading(TiXmlElement* pElement, unsigned int indent) = 0;
	/*!
	 * This method returns the indent
	 * @param numIndents [unsigned int] number of indents
	 */
	const char* getIndent( unsigned int numIndents );
	static const unsigned int NUM_INDENTS_PER_SPACE = 2;

	TiXmlDocument doc;
	bool loadOkay;
};