#pragma once
#include "glm/glm.hpp"
#include "Camera.h"
#include "View.h"
#include "Light.h"
#include "RenderImplementation.h"

#include <glew.h>
#include <QGLWidget>
#include <QtOpenGL>
#include <QColor>
#include <QGLContext>
#include <QGLFormat>
#include <vector>

/*!
 * This class renders the tree, using the Renderimplementation class.
 * @author Caroline and Michael
 */
class OpenGLWidget : public QGLWidget
{
	Q_OBJECT

public:
	/*!
	 * Constructor which initialize this widget
	 */
	OpenGLWidget(QWidget* parent = 0);
	/*!
	 * Standarddeconstructor
	 */
	~OpenGLWidget();

	static QGLContext* context;
	static QGLFormat* format;

protected:
	/*!
	 * This method initialize OpenGL
	 */
	void initializeGL();
	/*!
	 * This method resize the OpengGL window
	 * @param width[int] width of the window
	 * @param height[int] height of the window
	 */
	void resizeGL(int width,int height);
	/*!
	 * This method paints the acutual frame
	 */
	void paintGL();

	/*!
	 * This metod gets called, if the mouse is pressed.
	 * @param event Mouse Event
	 */
    void mousePressEvent(QMouseEvent *event);
	/*!
	 * This metod gets called, if the mouse is moved.
	 * @param event Mouse Event
	 */
    void mouseMoveEvent(QMouseEvent *event);
	/*!
	 * This metod gets called, if the mouse is released.
	 * @param event Mouse Event
	 */
    void mouseReleaseEvent(QMouseEvent *event);

	/*!
	 * This metod gets called, if a keyboard button is pressed.
	 * @param event Mouse Event
	 */
	void keyPressEvent(QKeyEvent *event);
	/*!
	 * This metod gets called, if a keyboard button is released.
	 * @param event Mouse Event
	 */
	void keyReleaseEvent(QKeyEvent *event);

private:
	vec3 camera_uvt;
	float translateZ;


	bool mode3D;

	vec3 m_oldMousePosition;

	/** Clear color is black */ 
 	 QColor black;
	 /** The width of the screen */
	 int screenWidth;
	 /** The height of the screen */
	 int screenHeight;
	 //model matrix
	 glm::mat4 model;
	 //perspective or orthogonal view
	 bool perspective;

	 Light* m_light;

	 /** The camera. */
	 Camera* camera_orth;
	 Camera* camera_pers;
	 View** m_view;

	 /*!
	  * This method sets the camera to the projection mode
	  */
	 void setCameraProjection();

	 RenderImplementation* renderer;

private slots:
	void setView(View** view);
};