#pragma once
#include "glm/glm.hpp"

using glm::vec3;
using glm::vec4;
using glm::mat4;

/*!
 * This class saves the actual states from the UI, and can read at every position of the application.
 * @author Michael Beham
 * @date 3.6.2011
 */
class RenderStates
{
public:
	/*!
	 * Standard constructor
	 */
	RenderStates(void);
	/*!
	 * Standard deconstructor
	 */
	~RenderStates(void);

	static int drawNodes;

	/*!
	 * 3d mode?
	 */
	static bool mode3d;
	/*!
	 * actual view
	 */
	static int actView;
	/*!
	 * shadows enable?
	 */
	static bool enable_shadow;
	/*!
	 * blending mode
	 */
	static int blendingMode;
	/*!
	 * blending on?
	 */
	static bool blendingon;

	/*!
	 * light attenuation 
	 */
	static int LightAttenuate;
	/*! 
	 * light radius
	 */
	static int LightRadius;
	/*! 
	 * light on
	 */
	static bool lighton;
	/*!
	 * light type
	 */
	static int lighttype;
	/*! 
	 * light position
	 */
	static vec3 lightpos;
	/*!
	 * Spot direction
	 */
	static vec3 spotdir;
	/*!
	 * shadow
	 */
	static int shadow;

	/*!
	 * X Rotation of the tree
	 */
	static int RotateX;
	/*!
	 * Y Rotation of the tree
	 */
	static int RotateY;
	/*!
	 * Z Rotation of the tree
	 */
	static int RotateZ;
	/*!
	 * Scale of the tree
	 */
	static int Scale;

	/*!
	 * Select the treemap layout type
	 */
	static int treemaptype; //0...slice and dice 1...strips
	/*!
	 * Should the cushion texture draw?
	 */
	static bool cushionTexture;

	/*!
	 * Should the edges mirrors?
	 */
	static bool mirrorRelationEdges;

	/*!
	 * RGB value of the vertex color
	 */
	static vec3 vertexColor;
	/*!
	 * alpha Value of the vertex color
	 */
	static float vertexAlpha;
	/*!
	 * RGB value of the hierarchy color
	 */
	static vec3 hierarchicColor;
	/*!
	 * alpha Value of the hierarchycolor
	 */
	static float herarchicAlpha;
	/*!
	 * RGB value of the startrelation color
	 */
	static vec3 startRelationColor;
	/*!
	 * alpha Value of the startrelationcolor
	 */
	static float startRelationAlpha;
	/*!
	 * RGB value of the endrelation color
	 */
	static vec3 endRelationColor;
	/*!
	 * alpha Value of the endrelationcolor
	 */
	static float endRelationAlpha;
	/*!
	 * RGB value of the selectioncolor
	 */
	static vec3 selectionColor;
	/*!
	 * Alphavalue of the selctioncolor
	 */
	static float selectionAlpha;

	/*!
	 * Beta of B-Spline curve
	 */
	static float beta;
};
