#pragma once
#include "BSpline.h"
#include "Shader.h"
#include "FBO.h"
#include "Camera.h"
#include "Light.h"
#include "View.h"
#include "Image.h"
#include "Text.h"
#include "RenderRadialTree.h"
#include "RenderBallonTree.h"
#include "RenderRootedTree.h"
#include "RenderTreeMap.h"
#include "SelectedObject.h"
#include "RenderStates.h"
#include "SplineSet.h"
#include <QVector>
#include <QVector3D>

#include "glm/glm.hpp"
#include <vector>
#include "Matrix.h"

class Renderer
{
public:
	enum BlendingMode
	{
		STD_BLENDING,
		MAX_BLENDING,
		STDMAX_BLENDING,
		AVGWEIGHTING_BLENDING,
		SUMAVG_BLENDING
	};

public:
	/*!
	 * standard constructor
	 */
	Renderer(void);
	/*!
	 * standard deconstructor
	 */
	~Renderer(void);

	/*!
	 * This method updates the node's color
	 *@param color [vec3] new color of the nodes
	 *@param alpha [float] new alpha value of the nodes
	 */
	virtual void UpdateSphereColor(vec3 color, float alpha) = 0;
	/*!
	 * This method updates the hierarchy edges color
	 *@param color [vec3] new color of the hierarchy edges
	 *@param alpha [float] new alpha value of the hierarchy edges
	 */
	virtual void UpdateHierarchyEdgeColor(vec3 color, float alpha) = 0;
	/*!
	 * This method updates the relation edges color
	 *@param color [vec3] new color of the relation edges
	 *@param alpha [float] new alpha value of the relation edges
	 */
	virtual void UpdateRelationEdgeColor(vec3 color, float alpha, vec3 endcolor, float endalpha) = 0;
	/*!
	 * This method updates the selction color
	 *@param color [vec3] new color of the selction
	 *@param alpha [float] new alpha value of the selction
	 */
	virtual void UpdateSelectionColor(vec3 color, float alpha) = 0;

	/*!
	 * This method enables the blending
	 * @param value [bool] true... enable blending false... disable blending
	 */
	void SetBlending(bool value);
	/*!
	 * This method defines which elements should be drawn. 
	 * The input follows the idea of a bit array.
	 * @li 1... nodes should be  draw
	 * @li 2... hierarchy edges should be draw
	 * @li 4.... relation edges should be draw
	 * @li 8.... grid should be draw
	 * @param elements [int] elements which should be draw.
	 */
	void SetEnabledElements(int elements);
protected:
	/*!
	 * This method updates all states.
	 * It reads the RenderStates class for the new states
	 */
	void UpdateStates();

	Light* m_light;
	SelectedObject* selectedObj;
	
	int actView; 
	int blendingmode;

	vec3 rotateTree;
	float scaleTree;

	RenderTree* tree[5];
	View** m_view;

	vec3 sphereColor;
	float sphereAlpha;
	vec3 hierarchyColor;
	float hierarchyAlpha;
	vec3 relationStartColor;
	float relationStartAlpha;
	vec3 relationEndColor;
	float relationEndAlpha;
	vec3 selectionColor;
	float selectionAlpha;

	bool m_blending;
	bool m_drawRelationEdges;
	bool m_drawHierarchicEdges;
	bool m_drawVertices;
	bool m_drawGrid;
	bool m_mirrorrelationedges;

	//B-Splines
	unsigned int numberofhierarchyedges;
	SplineSet* splineSet;
	int numberofHierarchies;
	float* controlPolygonX;
	float* controlPolygonY;
	float m_beta;
	int numreledges;	
};
