#pragma once
#include "View.h"

/*!
 * Diese Klasse speichert des aktuell selektierte Objekt.
 * @author Michael Beham
 * @date 3.6.2011
 */
class SelectedObject
{
public:
	/*!
	 * Definiert den aktuell selektierten Typ
	 */
	enum ObjType
	{
		SO_Vertex,
		SO_HEDGE,
		SO_REDGE,
		SO_NONE
	};

public:
	/*!
	 * Standardkonstruktor
	 */
	SelectedObject(void);
	/*!
	 * Standarddekonstruktor
	 */
	~SelectedObject(void);

	/*!
	 * Standardkonstruktor welches ein VertexPtr als aktuell selektiertes Objekt die bergebene Vertex definiert.
	 * @param [VertexPtr] obj Node welches selektiert werden soll.
	 */
	SelectedObject(VertexPtr obj);
	/*!
	 * Standardkonstruktor welches ein HierarchyPtr als aktuell selektiertes Objekt die bergebene HierarchyPtr definiert.
	 * @param [HierarchyPtr] obj Edge welches selektiert werden soll.
	 */
	SelectedObject(HierarchyPtr obj);
	/*!
	 * Standardkonstruktor welches ein RelationEdge als aktuell selektiertes Objekt die bergebene RelationEdge definiert.
	 * @param [RelationPtr] obj Edge welches selektiert werden soll.
	 */
	SelectedObject(RelationPtr obj);

	/*!
	 * Diese Funktion setzt den bergebenen VertexPtr als aktuell selektiertes Objekt.
	 * @param [VertexPtr] obj aktuell selektiertes Objekt
	 */
	void SetSelectedObj(VertexPtr obj);
	/*!
	 * Diese Funktion setzt den bergebenen HierarchyPtr als aktuell selektiertes Objekt.
	 * @param [HierarchyPtr] obj aktuell selektiertes Objekt
	 */
	void SetSelectedObj(HierarchyPtr obj);
	/*!
	 * Diese Funktion setzt den bergebenen RelationPtr als aktuell selektiertes Objekt.
	 * @param [RelationPtr] obj aktuell selektiertes Objekt
	 * @param [int] helper Liefert die Position im BSpline der aktuell selektierten Objekt
	 */
	void SetSelectedObj(RelationPtr obj, int helper);

	/*!
	 * Diese Funktion den Typ des aktuell selektierten Objektes.
	 * @return [int] Typ des aktuell selektierten Objekt
	 */
	int GetActualSelectedObjType();
	/*!
	 * Diese Funktion die selektierte Vertex
	 * @return [VertexPtr] selektiertes Vertex
	 */
	VertexPtr GetVertex();
	/*!
	 * Diese Funktion die selektierte HierarchyEdge
	 * @return [HierarchyPtr] selektiertes HierarchyEdge
	 */
	HierarchyPtr GetHierarchyEdge();
	/*!
	 * Diese Funktion die selektierte RelationEdge
	 * @return [RelationPtr] selektiertes RelationEdge
	 */
	RelationPtr GetRelationEdge();
	/*!
	 * Liefert des vorletzte selektierte HierarchyEdgeObjekt, damit dieses wieder auf die Standardfarbe gesetzt werden kann.
	 * @return [HierarchyPtr] vorletztes selektiertes Objekt.
	 */
	HierarchyPtr GetOldHierarchyEdge();
	/*!
	 * Liefert des vorletzte selektierte RelationEdgeObjekt, damit dieses wieder auf die Standardfarbe gesetzt werden kann.
	 * @return [RelationPtr] vorletztes selektiertes Objekt.
	 */
	RelationPtr GetOldRelationEdge();

	int m_oldhelper;
	int m_helper;
private:
	int obj_type;
	VertexPtr vertex_obj;
	HierarchyPtr hierarchy_obj;
	RelationPtr relation_obj;
	HierarchyPtr old_hierarchy_obj;
	RelationPtr old_relation_obj;
};
