#pragma once
#include "AbstractShader.h"
#include "ShaderObj.h"
#include <vector>

//!Shader class
/*!
 * Diese Klasse dient zur Verwaltung der Shader fr das Raycasting. 
 * Sie erbt die Funktionen von AbstractShader und erstellt, startet, stopt und lscht die Shader, welche mit Hilfe der Funktionen von Abstract Shader erstellt wurden.
 * @author Michael Beham
 * @date 4.12.2010
*/
class Shader : public AbstractShader
{
public:
	/*!
	 * Konstruktor: Erstellt alle Shader fr das Raycasting
	 */
	Shader(ShaderObj* shaderobj)
	{
		gTransformQuadric = createShader(shaderobj->vertexshader,shaderobj->fragmentshader, shaderobj);

		//glDeleteShader(vertexShaderHandle);
		//glDeleteShader(fragmentShaderHandle);
	};

	/*!
	 * Lscht alle Shaderobjekte, welche erstellt wurden
	 */
	~Shader()
	{
		glDeleteShader(gTransformQuadric->shaderID);
	}

	/*!
	 * Diese Funktion erstellt den jeweiligen Shader und verlinkt die ShaderID, sowie dessen Attribute.
	 * @param [in] vertexshadername Pfad zum Vertexshader
	 * @param [in] fragmentshadername Pfad zum Fragmentshader
	 * @return Verlinktes Shaderobjekt
	 */
	ShaderObj* createShader(const char* vertexshadername, const char* fragmentshadername, ShaderObj* shaderID)
	{
		GLhandleARB vertexShaderHandle;
		GLhandleARB fragmentShaderHandle;
		vertexShaderHandle   = loadShader(vertexshadername,GL_VERTEX_SHADER);
		fragmentShaderHandle = loadShader(fragmentshadername,GL_FRAGMENT_SHADER);
		shaderID->shaderID = glCreateProgramObjectARB();
		glAttachObjectARB(shaderID->shaderID,vertexShaderHandle);
		glAttachObjectARB(shaderID->shaderID,fragmentShaderHandle);
		glLinkProgramARB(shaderID->shaderID);

		for(unsigned int i = 0; i  < shaderID->parameters.size(); i++)
		{
			shaderID->parameters[i]->id = glGetUniformLocationARB(shaderID->shaderID, shaderID->parameters[i]->var);
		}
	
		return shaderID;
	}
	
	/*!
	 * Diese Funktion startet den Shader.
	 * @param [in] shadingmode Welche Renderingvariante soll bentzt werden (Contour, Standard, Shadow, Exploration?)
	 * @param [in] slidingx X Koordinate der Lichtquelle
	 * @param [in] slidingy Y Koordinate der Lichtquelle
	 * @param [in] slidingz Z Koordinate der Lichtquelle
	 * @param [in] slidemode Soll nur Slices gezeichnet werden.
	 * @param [in] slicing Bestimmt darzustellenden Slice
	 * @param [in] slidevalue Bestimmt darzustellenden Slice
	 * @param [in] central_differences Sollen die Gradienten mithilfe der Central Differences berechnet werden.
	 * @param [in] maxintensity Wie sollen die Ray abgetastet werden (MIP, Standard, First, Avg)
	 * @param [in] preshading Soll das Objekt geshadet werden (mithilfe von Phong)
	 * @param [in] interpolation Sollen die Werte interpoliert werden
	 * @param [in] resolution Auflsung des FBOs
	 * @param [in] steps Anzahl der Schritte
	 * @param [in] adaptiv Soll adaptiv gesamplt werden
	 */
	void start(ShaderObj* obj)
	{
		ShaderObj* Shader = gTransformQuadric;
		glUseProgramObjectARB(obj->shaderID);

		for(unsigned int i = 0; i < obj->parameters.size(); i++)
		{
			switch(obj->parameters[i]->type)
			{
			case ParameterObj::PT_INT: 
					glUniform1iARB(obj->parameters[i]->id, (int)obj->parameters[i]->value[0]);
				break;
			case ParameterObj::PT_FLOAT: 
					glUniform1fARB(obj->parameters[i]->id, obj->parameters[i]->value[0]);
				break;
			case ParameterObj::PT_INT2: 
					glUniform2iARB(obj->parameters[i]->id, (int)obj->parameters[i]->value[0], (int)obj->parameters[i]->value[1]);
				break;
			case ParameterObj::PT_FLOAT2: 
					glUniform2fARB(obj->parameters[i]->id, obj->parameters[i]->value[0], obj->parameters[i]->value[1]);
				break;
			case ParameterObj::PT_INT3: 
				glUniform3iARB(obj->parameters[i]->id, (int)obj->parameters[i]->value[0], (int)obj->parameters[i]->value[1], (int)obj->parameters[2]->value[2]); 
				break;
			case ParameterObj::PT_FLOAT3: 
					glUniform3fARB(obj->parameters[i]->id, obj->parameters[i]->value[0], obj->parameters[i]->value[1], obj->parameters[2]->value[2]); 
				break;
			/*case ParameterObj::PT_MAT2: 
					glUniformMatrix2fvARB(obj->parameters[i]->id, &obj->parameters[i]->value[0]);
				break;
			case ParameterObj::PT_MAT3: 
					glUniformMatrix3fvARB(obj->parameters[i]->id, &obj->parameters[i]->value[0]);
				break;
			case ParameterObj::PT_MAT4: 
					glUniformMatrix4fvARB(obj->parameters[i]->id, &obj->parameters[i]->value[0]);
				break;*/
			}
		}
	};

	/*!
	* Diese Funktion stoppt den Shader
	*/
	void stop()
	{
		glUseProgramObjectARB(0);
	};

private:
	ShaderObj* gTransformQuadric;
};
