#pragma once

#include "BSpline.h"
#include <QVector>
#include <QVector3D>
#include <glm/glm.hpp>

 /*!
  * This class organizes the B-Spline objects
  */ 
class SplineSet
{
public: 
	/*!
	*	Constructs a new SplineSet object the parameter needed is the numberOfHierarchy for the unerlaying Basisfunktions
	*/
	SplineSet(int numberOfHierarchy);
	/*!
	*	Destructor
	*/
	~SplineSet();

	/*!
	*	adds a Bspline with the given controlpolygon to the view-corresponding QVector 
	* @param polygon a pointer to the object with the controlpoints of the controlpolygon
	* @param size the number of control points
	* @param view determines to which QVector the B-Spline is added
	* @param id is the id of the adjaceny edge
	*/
	void addBSpline(QVector<QVector3D>* polygon, int size, int view, float id);
	/*!
	*	calls the recalculate-method of the B-Spline objects in all QVectors
	* @param bundlingBeta is the bundling strength and is between 0.0f and 1.0f
	*/
	void setNewBundlingValue(float bundlingBeta);
	/*!
	*	return a pointer to the Vector with pointer to all the B-Splines corresponding to the view
	* @param view indicates which QVector should be returned by these Method
	* @return a pointer to the specified QVector
	*/
	QVector<BSpline*>* getSplineVector(int view);
	/*!
	* sorts the B-Splines in a QVector ascending by length
	*/
	void sortBSpline();

private:

	/*! QVector containing all BSplines of the rooted-tree view*/
	QVector<BSpline*>* rootedBSplineVector;
	/*! QVector containing all BSplines of the treeMap Slice and Dice-tree view*/
	QVector<BSpline*>* treeMapSDBSplineVector;
	/*! QVector containing all BSplines of the balloon-tree view*/
	QVector<BSpline*>* ballonBSplineVector;
	/*! QVector containing all BSplines of the treeMap Strip-tree view*/
	QVector<BSpline*>* TreeMapStripSplineVector;
	QVector<BSpline*>* RadialTreeSplineVector;

	/*! a float value between 0.0f and 1.0f giving the bundling strength for the B-Splines*/ 
	float bundlingBeta;
	/*! the number of Hierarchies to initialize the Basis-functions*/ 
	int numberOfHierarchies;
	/*! the number of points which are calculated for the line-strip of the B-Spline*/ 
	int pointsPerSpline;
	/*! Array with the Basis-functions for the Splines */ 
	BasisFunktion** basisFunktionArray;

};