#include "BSpline.h"
#include <glew.h>
#include <QtOpenGL>
#include <gl\gl.h>			
#include <gl\glu.h>	
#include <iostream>

BSpline::BSpline(BasisFunktion* basisFunktion, float* controlPolygonX, float* controlPolygonY,float* controlPolygonZ, int controlPolygonSize, float id) : m_id(id)
{
	this->basisFunktion = basisFunktion;
	this->initialX = controlPolygonX;
	this->initialY = controlPolygonY;
	this->initialZ = controlPolygonZ;

	this->controlPolygonSize = controlPolygonSize;

	bundlingX = new float[controlPolygonSize];
	bundlingY = new float[controlPolygonSize];
	bundlingZ = new float[controlPolygonSize];

	for ( int i = 0; i < controlPolygonSize; i++)
	{
		bundlingX[i] = initialX[i];
		bundlingY[i] = initialY[i];
		bundlingZ[i] = initialZ[i];
		//std::cout << "init  x: " << initialX[i] << "y: " << initialY[i] << std::endl; 
	}
	splinePoints = basisFunktion->getNumberSplinePoints();
	values = new vec3[splinePoints];

	length = 0.0f;

	calculate();

	vec4 startcolor = vec4(0.0f, 1.0f, 0.0f,0.2f);
	vec4 endcolor = vec4(0.0f, 0.0f, 1.0f,0.2f);

	vec4* hieararchycolor = new vec4[splinePoints];
	vec4* hieararchypicking = new vec4[splinePoints];
	for(int i = 0; i < splinePoints; i++)
	{
		float weight = (float)(i)/(float)(splinePoints);

		hieararchycolor[i] = vec4((1.0f-weight)*startcolor.x + weight * endcolor.x, (1.0f-weight)*startcolor.y + weight * endcolor.y, (1.0f-weight)*startcolor.z + weight * endcolor.z, 1.0/(controlPolygonSize));
		hieararchypicking[i] = vec4(0.0f,0.0f, m_arraypos, 0.0f);
	}

	glEnableClientState( GL_VERTEX_ARRAY );				// Disable Vertex Arrays
	glEnableClientState( GL_COLOR_ARRAY );				// Disable Color Arrays
	glEnableClientState( GL_TEXTURE_COORD_ARRAY );		// Disable Vertex Arrays

	// Generate And Bind The Vertex Buffer
	glGenBuffers( 1, &vbohierarchylines_vert );							// Get A Valid Name
	glBindBuffer( GL_ARRAY_BUFFER, vbohierarchylines_vert );			// Bind The Buffer
	glBufferData( GL_ARRAY_BUFFER, splinePoints*sizeof(vec3), values, GL_DYNAMIC_DRAW_ARB );	// Load The Data

	glGenBuffers( 1, &vbohierarchylines_color );							// Get A Valid Name
	glBindBuffer( GL_ARRAY_BUFFER, vbohierarchylines_color );			// Bind The Buffer
	glBufferData( GL_ARRAY_BUFFER, splinePoints*sizeof(vec4), hieararchycolor, GL_DYNAMIC_DRAW_ARB );	// Load The Data

	glGenBuffersARB( 1, &vboindex_picking_ind );							// Get A Valid Nam
	glBindBufferARB( GL_ARRAY_BUFFER, vboindex_picking_ind );			// Bind The Buffere
	glBufferDataARB( GL_ARRAY_BUFFER, splinePoints*sizeof(vec4), hieararchypicking, GL_DYNAMIC_DRAW_ARB);
				
	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_COLOR_ARRAY );				
	glDisableClientState(GL_TEXTURE_COORD_ARRAY );

	delete [] hieararchycolor;
	delete [] hieararchypicking;
}

BSpline::~BSpline(void)
{
	if(initialX) delete [] initialX;
	if(initialY) delete [] initialY;
	if(initialZ) delete [] initialZ;
	if(bundlingX) delete [] bundlingX;
	if(bundlingY) delete [] bundlingY;
	if(bundlingZ) delete [] bundlingZ;
	if(values) delete [] values;
}

void BSpline::SetID(int id)
{
	m_id = id;
}

void BSpline::SetVectorPos(int id)
{
	m_arraypos = id;
}

int BSpline::GetID()
{
	return m_id;
}
int BSpline::GetVectorPos()
{
	return m_arraypos;
}

void BSpline::calculate()
{
	float x;
	float y;
	float z;

	for (int i = 0; i < splinePoints; i++)
	{
		x = 0.0f;
		y = 0.0f;
		z = 0.0f;

		for(int j = 0; j < controlPolygonSize; j++)
		{
			x += bundlingX[j] * basisFunktion->getValue(j, i);
			y += bundlingY[j] * basisFunktion->getValue(j, i);
			z += bundlingZ[j] * basisFunktion->getValue(j, i);
			//std::cout << "Basisfunktion getvalue at [" << j << "][" << i << "]" << basisFunktion->getValue(j, i) << std::endl;
		}
		
		values[i].x = x;
		values[i].y = 1.0 - y;
		values[i].z = z;
		
		//std::cout << "calculated Bspline x: " << x << "  y: " << y << "  z: " << z <<std::endl; 
	}
	calculateLength();
}

void BSpline::recalculate(float beta)
{
	for (int i = 0; i < controlPolygonSize; i++)
	{
		bundlingX[i] = beta * initialX[i] + (1.0f - beta) * (initialX[0] + 
			static_cast<float>(i / (controlPolygonSize - 1)) * (initialX[controlPolygonSize - 1] - initialX[0]));

		bundlingY[i] = beta * initialY[i] + (1.0f - beta) * (initialY[0] +
			static_cast<float>(i / (controlPolygonSize - 1)) * (initialY[controlPolygonSize - 1] - initialY[0]));

		bundlingZ[i] = beta * initialZ[i] + (1.0f - beta) * (initialZ[0] +
			static_cast<float>(i / (controlPolygonSize - 1)) * (initialZ[controlPolygonSize - 1] - initialZ[0]));

	}

	calculate();
	UpdatePosition();
}

void BSpline::UpdatePosition()
{
	// bind then map the VBO
	glBindBufferARB(GL_ARRAY_BUFFER, vbohierarchylines_vert);
	float* ptr = (float*)glMapBufferARB(GL_ARRAY_BUFFER, GL_WRITE_ONLY_ARB);

	// if the pointer is valid(mapped), update VBO
	if(ptr)
	{	
		for(int i = 0; i < splinePoints; i++)
		{
			ptr[(3*i)+0] = values[i].x;
			ptr[(3*i)+1] = values[i].y;
			ptr[(3*i)+2] = values[i].z;
		}
		glUnmapBufferARB(GL_ARRAY_BUFFER); // unmap it after use
	}
}

void BSpline::UpdateColor(vec4 startcolor, vec4 endcolor)
{
	vec4* hieararchycolor = new vec4[splinePoints];
	for(int i = 0; i < splinePoints; i++)
	{
		float weight = (float)(i)/(float)(splinePoints);
		//hieararchycolor[i] = vec4((1.0f-weight)*startcolor.x + weight * endcolor.x, (1.0f-weight)*startcolor.y + weight * endcolor.y, (1.0f-weight)*startcolor.z + weight * endcolor.z, (1.0f-weight)*startcolor.w +  weight * endcolor.w);
		hieararchycolor[i] = vec4((1.0f-weight)*startcolor.x + weight * endcolor.x, (1.0f-weight)*startcolor.y + weight * endcolor.y, (1.0f-weight)*startcolor.z + weight * endcolor.z, startcolor.w/(controlPolygonSize));
	}

	// bind then map the VBO
	glBindBufferARB(GL_ARRAY_BUFFER, vbohierarchylines_color);
	float* ptr = (float*)glMapBufferARB(GL_ARRAY_BUFFER, GL_WRITE_ONLY_ARB);

	// if the pointer is valid(mapped), update VBO
	if(ptr)
	{	
		for(unsigned int i = 0; i < splinePoints; i++)
		{
			ptr[(4*i)+0] = hieararchycolor[i].x;
			ptr[(4*i)+1] = hieararchycolor[i].y;
			ptr[(4*i)+2] = hieararchycolor[i].z;
			ptr[(4*i)+3] = hieararchycolor[i].w;
		}
		glUnmapBufferARB(GL_ARRAY_BUFFER); // unmap it after use
	}
	delete [] hieararchycolor;
}

void BSpline::UpdateIDs(float id)
{
	vec4* hieararchypicking = new vec4[splinePoints];
	for(int i = 0; i < splinePoints; i++)
	{
		float weight = (float)(i)/(float)(splinePoints);
		hieararchypicking[i] = vec4(0.0f,0.0f, id, 0.0f);
	}

	// bind then map the VBO
	glBindBufferARB(GL_ARRAY_BUFFER, vboindex_picking_ind);
	float* ptr = (float*)glMapBufferARB(GL_ARRAY_BUFFER, GL_WRITE_ONLY_ARB);

	// if the pointer is valid(mapped), update VBO
	if(ptr)
	{	
		for(unsigned int i = 0; i < splinePoints; i++)
		{
			ptr[(4*i)+0] = hieararchypicking[i].x;
			ptr[(4*i)+1] = hieararchypicking[i].y;
			ptr[(4*i)+2] = hieararchypicking[i].z;
			ptr[(4*i)+3] = hieararchypicking[i].w;
		}
		glUnmapBufferARB(GL_ARRAY_BUFFER); // unmap it after use
	}
	delete [] hieararchypicking;
}

void BSpline::calculateLength()
{	
	float diffX = 0.0f;
	float diffY = 0.0f;
	float diffZ = 0.0f;

	for (int i = 1; i < splinePoints; i++)
	{
		diffX = values[i].x - values[i-1].x;
		diffY = values[i].y - values[i-1].y;
		diffZ = values[i].z - values[i-1].z;
		length += diffX * diffX + diffY * diffY + diffZ * diffZ;
	}

}
vec3* BSpline::getValues()
{
	if(values)
	{
		return values;
	}
	else
	{
		return NULL;
	}
}

float BSpline::getLength() const
{
	return length;
}

void BSpline::draw()
{
	bool noshadow = true;
	//Draw Hierarchy edges
	glEnableClientState(GL_VERTEX_ARRAY);
	glEnableClientState(GL_COLOR_ARRAY);
	if(noshadow) glEnableClientState(GL_TEXTURE_COORD_ARRAY);

	glBindBuffer( GL_ARRAY_BUFFER, vbohierarchylines_vert );
	glVertexPointer( 3, GL_FLOAT, 0, 0 );		// Set The Vertex Pointer To The Vertex Buffer
	glBindBuffer(GL_ARRAY_BUFFER, vbohierarchylines_color);
	glColorPointer(4,GL_FLOAT, 0, 0);
		
	if(noshadow)
	{
		glActiveTexture(GL_TEXTURE7);
		glBindBuffer(GL_ARRAY_BUFFER, vboindex_picking_ind);
		glTexCoordPointer(4, GL_FLOAT, 0, 0);
	}
	glPushMatrix();
	glDrawArrays( GL_LINE_STRIP, 0, splinePoints);
	glPopMatrix();

	glDisableClientState(GL_VERTEX_ARRAY);
	glDisableClientState(GL_COLOR_ARRAY);
	if(noshadow) glDisableClientState(GL_TEXTURE_COORD_ARRAY);
}
/*/
#include "BSpline.h"
#include <QtOpenGL>
#include <gl\gl.h>           
#include <gl\glu.h>  
#include <iostream>
 
BSpline::BSpline(BasisFunktion* basisFunktion, float* controlPolygonX, float* controlPolygonY,float* controlPolygonZ, int controlPolygonSize)
{
    this->basisFunktion = basisFunktion;
    this->initialX = controlPolygonX;
    this->initialY = controlPolygonY;
    this->initialZ = controlPolygonZ;
 
    this->controlPolygonSize = controlPolygonSize;
 
    bundlingX = new float[controlPolygonSize];
    bundlingY = new float[controlPolygonSize];
    bundlingZ = new float[controlPolygonSize];
 
    for ( int i = 0; i < controlPolygonSize; i++)
    {
        bundlingX[i] = initialX[i];
        bundlingY[i] = initialY[i];
        bundlingZ[i] = initialZ[i];
        //std::cout << "init  x: " << initialX[i] << "y: " << initialY[i] << std::endl;
    }
    splinePoints = basisFunktion->getNumberSplinePoints();
    xValues = new float[splinePoints];
    yValues = new float[splinePoints];
    zValues = new float[splinePoints];
 
    length = 0.0f;
 
    calculate();
}
 
BSpline::~BSpline(void)
{
    if(initialX) delete [] initialX;
    if(initialY) delete [] initialY;
    if(initialZ) delete [] initialZ;
    if(bundlingX) delete [] bundlingX;
    if(bundlingY) delete [] bundlingY;
    if(bundlingZ) delete [] bundlingZ;
    if(xValues) delete [] xValues;
    if(yValues) delete [] yValues;
    if(zValues) delete [] zValues;
 
}
 
void BSpline::calculate()
{
    float x;
    float y;
    float z;
 
    for (int i = 0; i < splinePoints; i++)
    {
        x = 0.0f;
        y = 0.0f;
        z = 0.0f;
 
        for(int j = 0; j < controlPolygonSize; j++)
        {
            x += bundlingX[j] * basisFunktion->getValue(j, i);
            y += bundlingY[j] * basisFunktion->getValue(j, i);
            z += bundlingZ[j] * basisFunktion->getValue(j, i);
            //std::cout << "Basisfunktion getvalue at [" << j << "][" << i << "]" << basisFunktion->getValue(j, i) << std::endl;
        }
         
        xValues[i] = x;
        yValues[i] = 1.0 - y;
        zValues[i] = z;
         
        //std::cout << "calculated Bspline x: " << x << "  y: " << y << "  z: " << z <<std::endl;
    }
    calculateLength();
}
 
void BSpline::recalculate(float beta)
{
    for (int i = 0; i < controlPolygonSize; i++)
    {
        bundlingX[i] = beta * initialX[i] + (1.0f - beta) * (initialX[0] +
            static_cast<float>(i / (controlPolygonSize - 1)) * (initialX[controlPolygonSize - 1] - initialX[0]));
 
        bundlingY[i] = beta * initialY[i] + (1.0f - beta) * (initialY[0] +
            static_cast<float>(i / (controlPolygonSize - 1)) * (initialY[controlPolygonSize - 1] - initialY[0]));
 
        bundlingZ[i] = beta * initialZ[i] + (1.0f - beta) * (initialZ[0] +
            static_cast<float>(i / (controlPolygonSize - 1)) * (initialZ[controlPolygonSize - 1] - initialZ[0]));
 
    }
 
    calculate();
}
 
void BSpline::calculateLength()
{  
    float diffX = 0.0f;
    float diffY = 0.0f;
    float diffZ = 0.0f;
 
    for (int i = 1; i < splinePoints; i++)
    {
        diffX = xValues[i] - xValues[i - 1];
        diffY = yValues[i] - yValues[i - 1];
        diffZ = zValues[i] - zValues[i - 1];
        length += diffX * diffX + diffY * diffY + diffZ * diffZ;
    }
 
}
 
float* BSpline::getXValues()
{
    if(xValues )
    {
        return xValues;
    }else{
        return NULL;
    }
     
}
 
float* BSpline::getYValues()
{
    if(yValues)
    {
        return yValues;
    }else{
        return NULL;
    }
}
 
float* BSpline::getZValues()
{
    if(zValues)
    {
        return zValues;
    }else{
        return NULL;
    }
}
 
float BSpline::getLength() const
{
    return length;
}
 
void BSpline::draw()
{
    glBegin(GL_LINE_STRIP);
 
    for (int i = 0; i < splinePoints; i++)
    {  
    glVertex3f(xValues[i], yValues[i], zValues[i]);
    }
 
    glEnd();
 
}//*/