#include "BallonTree.h"
#include <glm/gtc/matrix_transform.hpp> //translate, rotate, scale, perspective

VertexBallon::VertexBallon() {}
VertexBallon::VertexBallon(VertexPtr v) : m_pkV(v){}
VertexBallon::VertexBallon(VertexPtr v, double r, double rr, double circumference, double ark, double angle) : m_pkV(v), m_r(r), m_rr(rr), m_ark(ark), m_angle(angle), m_circumference(circumference){}
double VertexBallon::GetCircumference(){ return m_circumference;	}
VertexPtr VertexBallon::GetVertex() {	return m_pkV;	}
double VertexBallon::GetRadius() {	return m_r;	}
double VertexBallon::GetRadiusRendering() {	return m_rr; }
double VertexBallon::GetArk(){	return m_ark;	}
double VertexBallon::GetAngle(){	return m_angle;	}
void VertexBallon::SetVertex(VertexPtr v) {	m_pkV = v;	}
void VertexBallon::SetRadius(double r) {	m_r = r; }
void VertexBallon::SetRadiusRendering(double r) {	m_rr = r; }
void VertexBallon::SetArk(double ark){	m_ark = ark;	}
void VertexBallon::SetAngle(double angle){	m_angle = angle;	}
void VertexBallon::SetCircumference(double circumference){ m_circumference = circumference;	}


BallonTree::BallonTree()
{
}

BallonTree::~BallonTree(void)
{
}

void BallonTree::Calculate()
{
	std::vector<VertexPtr> verticesWithoutChilds = m_pkGraph->GetAllVerticesWithoutChild();

	std::cout << "Calculate Radius" << std::endl;
	for(unsigned int i = 0; i < verticesWithoutChilds.size(); i++)
	{
		VertexBallonPtr tmp(new VertexBallon(verticesWithoutChilds[i], 0.01f, 0.01f, 2.0*PI*0.01f, 0, 0));
		verticesChilds.insert(pair<int,VertexBallonPtr>(tmp->GetVertex()->GetID(), tmp));
	}
	
	for(int actLevel = m_pkGraph->GetNumberOfHierarchies()-1; actLevel > 0; actLevel--)
	{
		for(std::map<unsigned int, VertexBallonPtr> ::iterator iter = verticesChilds.begin(); iter != verticesChilds.end(); iter++ )
		{
			if(iter->second->GetVertex()->GetHierarchyLevel() == actLevel)
			{	
				std::map<unsigned int, VertexBallonPtr>::iterator it = verticesChilds.find(iter->second->GetVertex()->GetParent()->GetID()); 
				if(it != verticesChilds.end())	it->second->SetCircumference(it->second->GetCircumference() + (2.0*iter->second->GetRadius()));
				else			
				{
					VertexBallonPtr tmp(new VertexBallon(iter->second->GetVertex()->GetParent(), 0, 0, 2.0*iter->second->GetRadius(), 0, 0));
					verticesChilds.insert(pair<unsigned int,VertexBallonPtr>(tmp->GetVertex()->GetID(), tmp));
				}
			}
		}

		for(std::map<unsigned int, VertexBallonPtr> ::iterator iter = verticesChilds.begin(); iter != verticesChilds.end(); iter++ )
		{
			if(iter->second->GetVertex()->GetHierarchyLevel() == actLevel-1)
			{	
				double num_of_childs = iter->second->GetVertex()->GetNumberChildVertices();
				double oldvalue = iter->second->GetCircumference();
				double oldvalue2 = oldvalue;

				double maxrad = 0;
				for(unsigned int i = 0; i < iter->second->GetVertex()->GetNumberChildVertices(); i++)	maxrad = max(maxrad, verticesChilds[iter->second->GetVertex()->GetChildVertexAt(i)->GetID()]->GetRadius());

				if(num_of_childs > 0)
				{
					//num_of_childs = 10.0/num_of_childs;	//Falls es wenige Childs gibt, damit ich den Circumference messen kann, dann multipliziere ich es noch mit nen Faktor
					num_of_childs = 10.0-num_of_childs;	//Falls es wenige Childs gibt, damit ich den Circumference messen kann, dann multipliziere ich es noch mit nen Faktor
					if(num_of_childs > 1.0 && num_of_childs <= 10.0)		oldvalue += num_of_childs*0.01;
					//if(num_of_childs > 1.0 && num_of_childs < 10.0)		oldvalue += num_of_childs*maxrad/2.f;
				}

				iter->second->SetRadiusRendering(oldvalue/(PI*2.0));
				iter->second->SetRadius(oldvalue/(PI*2.0)+maxrad);
				iter->second->SetCircumference(iter->second->GetRadius()*(2.0));
			}
		}
	}

	std::cout << "Calculate Angle" << std::endl;
	std::vector<VertexPtr> ptr;
	ptr.push_back(m_pkGraph->GetRoot());

	verticesChilds[m_pkGraph->GetRoot()->GetID()]->SetAngle(0);//*/
	while(ptr.size() > 0)
	{
		if(ptr[0]->GetNumberChildVertices() > 0)
		{
			verticesChilds[ptr[0]->GetChildVertexAt(0)->GetID()]->SetAngle(0.0f);			
			ptr.push_back(ptr[0]->GetChildVertexAt(0));

			double radius_parent = verticesChilds[ptr[0]->GetID()]->GetRadiusRendering();
			double radius_first = verticesChilds[ptr[0]->GetChildVertexAt(ptr[0]->GetNumberChildVertices()-1)->GetID()]->GetRadiusRendering();

			double num_of_childs = ptr[0]->GetNumberChildVertices();
			double oldvalue = 0.f;
			//double oldvalue = 1.f;
			if(num_of_childs > 0)
			{
				//num_of_childs = 10.0/(10.f-num_of_childs);	//Falls es wenige Childs gibt, damit ich den Circumference messen kann, dann multipliziere ich es noch mit nen Faktor
				num_of_childs = 10.0-num_of_childs;	//Falls es wenige Childs gibt, damit ich den Circumference messen kann, dann multipliziere ich es noch mit nen Faktor
				//num_of_childs = 10.0/num_of_childs;
				if(num_of_childs >= 1.0 && num_of_childs <= 10.0)	
				{
					if(ptr[0]->GetHierarchyLevel() == m_pkGraph->GetNumberOfHierarchies()-2)		oldvalue = 1.0*0.01*(num_of_childs)/(double)(ptr[0]->GetNumberChildVertices());
					else	oldvalue = 4.0*0.01*(num_of_childs)/(double)(ptr[0]->GetNumberChildVertices());
				}
				//if(num_of_childs >= 1.0 && num_of_childs < 10.0)		oldvalue = num_of_childs;
			}

			for(unsigned int i = 0; i < ptr[0]->GetNumberChildVertices(); i++)
			{
				int child_id = ptr[0]->GetChildVertexAt(i)->GetID();				
				double radius_next = verticesChilds[child_id]->GetRadiusRendering();

				verticesChilds[child_id]->SetAngle(((oldvalue+radius_next + radius_first)/(radius_parent)));	
				//verticesChilds[child_id]->SetAngle(oldvalue*((radius_next + radius_first)/(radius_parent)));	
				ptr.push_back(ptr[0]->GetChildVertexAt(i));
				radius_first = radius_next;
			}
		}
		ptr.erase(ptr.begin());
	}

	std::cout << "make it ready" << std::endl;
	VertexPtr actVert = m_pkGraph->GetRoot();

	std::vector<VertexPtr> childVertex;
	childVertex.push_back(actVert);

	float pi = PI;
	int actHierarchy = 0;
	float sumangle = 0;
	for(unsigned int i = 0; i < childVertex.size(); i++)
	{
		VertexPtr newVert = childVertex[i];
		if(newVert->GetHierarchyLevel() != actHierarchy)
		{
			sumangle = 0.0f;
			actHierarchy++;
		}

		double angle = verticesChilds[newVert->GetID()]->GetAngle();
		sumangle+=angle;

		vec4 pos;
		vec3 startpos (0.0,-1.0,0.0);
	
		if(newVert->GetParent() != 0 && newVert->GetID() != m_pkGraph->GetRoot()->GetID())	
		{
			double radius = verticesChilds[newVert->GetParent()->GetID()]->GetRadiusRendering();
			//if(newVert->GetNumberChildVertices() > 0) radius = verticesChilds[newVert->GetParent()->GetID()]->GetRadius()-verticesChilds[newVert->GetParent()->GetID()]->GetRadiusRendering();
			startpos = GetGUIVertexForView(newVert->GetParent()->GetID()).GetPosition();
			
			mat4 rotmat(1.0);
			mat4 m = glm::rotate(rotmat, sumangle*180.0f/pi, glm::vec3(0.0f, 1.0f, 0.0f));

			radius = verticesChilds[newVert->GetParent()->GetID()]->GetRadius();
			pos = vec4(0.0f,0.0f,radius,1.0f)*m;
		}

		SetGUIVertexForView(newVert->GetID(), GUIVertex( vec3(pos.x, pos.y, pos.z )	+ startpos + vec3(0,(float)(actHierarchy)/(float)m_pkGraph->GetNumberOfHierarchies(),0) ,vec3(1,0,0),0.8 ));
		for(unsigned int j = 0; j < newVert->GetNumberChildVertices(); j++)
		{
			childVertex.push_back(newVert->GetChildVertexAt(j));
		}
	}
}