#include "BasisFunktion.h"
#include <iostream>


BasisFunktion::BasisFunktion(int numberControlPoints, int splinePoints)
{
	this->numberControlPoints = numberControlPoints;
	this->splinePoints = splinePoints;

	valueArray = new double*[numberControlPoints];

	for (int i = 0; i < numberControlPoints; i++)
	{
		valueArray[i] = new double[splinePoints];
	}
	
	if(numberControlPoints == 3)
	{
		p = 3;
	}else{
		p = 3 + 1;
	}	

	n = numberControlPoints + p;
	
	//std::cout << "p: " << p << " n: " << n << std::endl;
	initKnotVector();
	initValueArray();
}

BasisFunktion::~BasisFunktion()
{
	for (int i = 0; i < numberControlPoints; i++)
	{
		if(valueArray[i]) delete [] valueArray[i];
	}
	if(valueArray) delete [] valueArray;

	if(knotVector) delete [] knotVector;
}

void BasisFunktion::initKnotVector()
{
	knotVector = new int[n];
	int index = p;
	int rest;
	int knotValue;

	int endValue = n - 2 * p + 1;

	for ( int i = 0; i < p; i++)
	{
		knotVector[i] = 0;
		knotVector[n - i - 1] = endValue;
	}

	if (n > 2 * p)
	{
		rest = n - 2 * p;
		knotValue = 1;
		index = p;
		//std::cout << "endvalue: " << endValue << " rest: " << rest << std::endl;
		for (int i = 0; i < rest; i++)
		{
			knotVector[index] = knotValue;
			knotValue++;
			index++;
		}
	}
// 	for (int i = 0; i < n ; i++)
// 	{
// 		std::cout << "Knotvektor at " << i << " :" << knotVector[i] << std::endl;
// 	}

}
/** 
*	Return the value
*	@ param i Index of the BasisFunktion
*	@ param splinePoint Index of the Splinepoint
*/
double BasisFunktion::getValue(int i, int splinePoint)
{
	if ( splinePoint < 0 || splinePoint >= splinePoints )
	{
		return 0.0;
	}

	return valueArray[i][splinePoint];

}

void BasisFunktion::initValueArray()
{
	double u;

	double inc = knotVector[n - 1] - knotVector[0];
	inc /= (splinePoints - 1);

	for ( int i = 0; i < numberControlPoints; i++)
	{
		u = knotVector[0];

		for (int j = 0; j < splinePoints; j++)
		{
			if(i == numberControlPoints - 1 && j == splinePoints - 1)
			{
				valueArray[i][j] = 1.0;
			}else{
				valueArray[i][j] = coxRecursion(i + 1, p, u);
			}
			//std::cout << "initValue at ["<< i << "][" << j <<"] " << valueArray[i][j] << std::endl;
			u += inc;
		}
	}
}

double BasisFunktion::coxRecursion(int i, int p, double u)
{
	if( p == 1)
	{
		if( u >= knotVector[i - 1] && u <= knotVector [i])
		{
			return 1.0;
		}
		return 0.0;
	}

	double zaehler;
	double nenner;

	double koeff1;
	double koeff2;

	// 1 Teil:
	
	zaehler = u - knotVector[i - 1];
	nenner = knotVector[(i - 1) + (p - 1)]  - knotVector[i - 1];// p - 1 da der grad um 1 kleiner ist

	if( nenner == 0.0)
	{
		koeff1 = 0.0;
	} else {
		koeff1 = zaehler/nenner;
	}

	// 2 Teil:

	zaehler = knotVector[ i + (p - 1)] - u; // da i schon 1 + 1 ist wird hier nichts mehr dazuaddiert
	nenner = knotVector[ i  + (p - 1)] - knotVector[i];

	if( nenner == 0.0)
	{
		koeff2 = 0.0;
	} else {
		koeff2 = zaehler/nenner;
	}

	//wenn einer der beiden Koeffizienten 0 ist fllt der Teil weg. bzw wenn beide 0 sind ist es 0
	
	if (koeff1 == 0.0 && koeff2 == 0.0)
	{
		return 0.0;
	}

	if(koeff1 == 0.0)
	{
		return (koeff2 * coxRecursion(i + 1, p - 1, u));
	}

	if(koeff2 == 0.0)
	{
		return (koeff1 * coxRecursion(i, p - 1, u));
	}
	return (koeff1 * coxRecursion(i, p - 1, u) +
		koeff2 * coxRecursion(i + 1, p - 1, u));

}

int BasisFunktion::getNumberSplinePoints()
{
	return splinePoints;
}