#include "Camera.h"
#include <glm/gtc/matrix_projection.hpp>
#include <glm/gtx/transform2.hpp>


Camera::Camera(float zNear, float zFar, float fovyAngle, float aspect) : screenWidth(1), screenHeight(1), zoomFactor(1.0f)
{
	viewingMatrix = glm::mat4(1.0f);
	this->nearplane = zNear;
	this->farplane = zFar;
	this->fovy = fovyAngle;
	this->perspective(zNear, zFar, fovyAngle, aspect);
}

Camera::Camera(float left, float right, float bottom, float top, float near, float far) : screenWidth(1), screenHeight(1), zoomFactor(1.0f)
{
	viewingMatrix = glm::mat4(1.0f);
	this->nearplane = near;
	this->farplane = far;
	this->orthogonal(left, right, bottom, top, near, far);
}


Camera::~Camera()
{
}

mat4 Camera::getViewingMatrix() const
{
	return this->viewingMatrix;
}

void Camera::setViewingMatrix(mat4 matrix)
{
	this->viewingMatrix = matrix;
}

mat4 Camera::getProjectionMatrix()
{
	return this->projectionMatrix;
}

const glm::vec3 Camera::getPosition() const
{
	glm::vec4 pos = glm::inverse(getViewingMatrix()) * glm::vec4(0,0,0,1);
	return glm::vec3(pos.x, pos.y, pos.z);
}

void Camera::lookAt(vec3 position, vec3 upVector, vec3 lookAtVector)
{
	this->viewingMatrix = glm::lookAt(position, lookAtVector, upVector);
}

void Camera::perspective(float zNear, float zFar, float fovy, float aspect)
{
	this->projectionMatrix = glm::perspective(fovy, aspect, zNear, zFar);
}

void Camera::orthogonal(float left, float right, float bottom, float top, float near, float far)
{
	this->projectionMatrix = glm::ortho(left, right, bottom, top, near, far);
}

void Camera::setScreenWidth(float width)
{
	this->screenWidth = width;
}

float Camera::getScreenWidth()
{
	return this->screenWidth;
}

void Camera::setScreenHeight(float height)
{
	this->screenHeight = height;
}

float Camera::getScreenHeight()
{
	return this->screenHeight;
}

void Camera::setNearplane(float near)
{
	this->nearplane = near;
}

void Camera::setFarplane(float far)
{
	this->farplane = far;
}

void Camera::updateOrthogonalView()
{
	float left = -float(this->screenWidth)/2.0f * this->zoomFactor;
	float right = float(this->screenWidth)/2.0f * this->zoomFactor;
	float top = float(this->screenHeight/2.0f) * this->zoomFactor;
	float bottom = -float(this->screenHeight/2.0f) * this->zoomFactor;

	this->orthogonal(left, right, bottom, top, this->nearplane, this->farplane);


	this->projectionMatrix = glm::scale(this->projectionMatrix, glm::vec3(1, -1, 1));
}

void Camera::updatePerspectiveView()
{
	this->perspective(this->nearplane, this->farplane, float(this->screenWidth)/float(this->screenHeight), this->fovy * this->zoomFactor);
}

void Camera::setZoomFactor(float zoom)
{
	this->zoomFactor = zoom;
}

float Camera::getZoomFactor()
{
	return this->zoomFactor;
}

void Camera::zoomIn()
{
	this->zoomFactor *= 0.95f;

}

void Camera::zoomOut()
{
	this->zoomFactor *= 1.25f;

}