#include "Graph.h"

Graph::Graph(void)
{
}

Graph::~Graph(void)
{
	m_vertices.clear();
	m_hierarchyedges.clear();
	m_relationedge.clear();
}

void Graph::AddVertex(VertexPtr vert)
{
	unsigned int id = vert->GetID();
	m_vertices.insert(pair<unsigned int,VertexPtr>(id, vert));
}

VertexPtr Graph::GetVertexWithID(unsigned int id)
{
	return m_vertices[id];
}

int Graph::GetNumberOfVertices()
{
	return m_vertices.size();
}

void Graph::RemoveVertexWithID(unsigned int id)
{
	m_vertices.erase(id);
}

void Graph::RemoveVertex(Vertex& vert)
{
	RemoveVertexWithID(vert.GetID());
}

void Graph::SetRoot(int at)
{
	m_pkRoot = GetVertexWithID(at);
}

VertexPtr Graph::GetRoot()
{
	return m_pkRoot;
}

HierarchyPtr Graph::GetHierarchyEdgeWithID(unsigned int id)
{
	return m_hierarchyedges[id];
}

RelationPtr Graph::GetRelationEdgeWithID(unsigned int id)
{
	return m_relationedge[id];
}

void Graph::AddHierarchicEdge(HierarchyPtr edge)
{
	VertexPtr p1 = edge->GetStartEdge();
	VertexPtr p2 = edge->GetEndEdge();
	
	m_vertices[edge->GetStartEdge()->GetID()]->AddChildVertex(p2);
	m_vertices[edge->GetEndEdge()->GetID()]->SetParentVertex(p1);
	
	unsigned int id = edge->GetID();
	m_hierarchyedges.insert(pair<unsigned int,HierarchyPtr>(id, edge));
}

unsigned int Graph::GetNumberOfHiearchicEdges()
{
	return m_hierarchyedges.size();
}

void Graph::AddRelationEdge(RelationPtr edge)
{
	if(edge->GetStartEdge() != edge->GetEndEdge()){
		unsigned int id = edge->GetID();
		m_relationedge.insert(pair<unsigned int,RelationPtr>(id, edge));
	}
}

unsigned int Graph::GetNumberOfRelationEdges()
{
	return m_relationedge.size();
}

RelationPtr Graph::GetRelationEdgeAt(unsigned int at)
{
	/*
	//Eigentlich wre das richtig, ist aber total langsam (O(n))
	//ID wird eh iterativ vergeben...
	std::map<unsigned int, RelationPtr>::iterator relobj = m_relationedge.begin();
	for(int i = 0; i < at; i++) relobj++;
	return relobj->second;
	/*/
	return m_relationedge[at];//*/
}

HierarchyPtr Graph::GetHierarchyEdgeAt(unsigned int at)
{
	/*
	//Eigentlich wre das richtig, ist aber total langsam (O(n))
	//ID wird eh iterativ vergeben...
	std::map<unsigned int, HierarchyPtr>::iterator relobj = m_hierarchyedges.begin();
	for(int i = 0; i < at; i++) relobj++;
	return relobj->second;
	/*/
	return m_hierarchyedges[at];//*/
}

void Graph::SetNumberOfHierarchies(int count)
{
	m_hierarchy.resize(count);
}

int Graph::GetNumberOfHierarchies()
{
	return m_hierarchy.size();
}

void Graph::AddHierarchy(Hierarchy hierarchy)
{
	m_hierarchy.push_back(hierarchy);
}

void Graph::SetHierarchyAt(int level, Hierarchy hierarchy)
{
	m_hierarchy[level] = hierarchy;
}

Hierarchy Graph::GetHierarchyAt(int level)
{
	return m_hierarchy.at(level);
}

void Graph::RemoveHierarchy(int level)
{
	m_hierarchy.erase(m_hierarchy.begin()+level, m_hierarchy.begin()+level);
}

std::vector<VertexPtr> Graph::GetVertexPtrVectorFromIDVector(std::vector<unsigned int> ids)
{
	std::vector<VertexPtr> path;
	path.resize(0);

	for(unsigned int i = 0; i  < ids.size(); i++)	path.push_back(GetVertexWithID(ids[i]));

	return path;
}

void Graph::Calculate()
{
//	std::sort(m_vertices.begin(), m_vertices.end());
	CalculateHierarchy();
	std::cout << "Hierarchy calculated" << std::endl;
	CalculatePathToRoot();
	std::cout << "Path to root calculated" << std::endl;
}

void Graph::CalculateHierarchy()
{
	std::cout << "calculate hierarchy" << std::endl;
	//Achtung: knnte evt. falsch sein

	int count = 0;
	int level = 0;

	std::vector<VertexPtr> helper;
	std::vector<VertexPtr> helper2;
	VertexPtr actVertex;
	m_hierarchy.push_back(Hierarchy(1));
	m_hierarchy.push_back(m_pkRoot->GetNumberChildVertices());
	helper.push_back(m_pkRoot);
	m_pkRoot->SetHierarchyLevel(level);

	std::cout << "before do" << std::endl;
	do
	{
		helper2.clear();
		count = 0;
		for(unsigned int j = 0; j < helper.size(); j++)
		{
			helper2.push_back(helper.at(j));
		}
		helper.clear();

		for(unsigned int j = 0; j < helper2.size(); j++)
		{
			actVertex = helper2[j];

			for(unsigned int i = 0; i < actVertex->GetNumberChildVertices(); i++)
			{
				helper.push_back(actVertex->GetChildVertexAt(i));
				count += actVertex->GetChildVertexAt(i)->GetNumberChildVertices();
				actVertex->GetChildVertexAt(i)->SetHierarchyLevel(level+1);
			}
		}

		std::cout << "att hierarchy: " << count << std::endl;
		m_hierarchy.push_back(Hierarchy(count));

		level++;
	}
	while(count > 0);

	m_hierarchy.pop_back();
	std::cout << "LEVEL: " << level << " m_hierarchy.size() " << m_hierarchy.size() <<  std::endl;
}

void Graph::CalculatePathToRoot()
{
	std::vector<unsigned int> path;
	path.resize(0);

	path.push_back(m_pkRoot->GetID());
	m_pkRoot->SetPathToRoot(path);

	for(unsigned int i = 0; i < m_pkRoot->GetNumberChildVertices(); i++)	CalculatePathToRoot(m_pkRoot->GetChildVertexAt(i),  path);
}

void Graph::CalculatePathToRoot(VertexPtr actPath, std::vector<unsigned int> path)
{
	path.insert(path.begin(), actPath->GetID());
	actPath->SetPathToRoot(path);

	for(unsigned int i = 0; i < actPath->GetNumberChildVertices(); i++)	CalculatePathToRoot(actPath->GetChildVertexAt(i),  path);
}

std::vector<unsigned int> Graph::CalculatePathOverLCA(RelationPtr edge)
{
	return CalculatePathOverLCA(edge->GetStartEdge(), edge->GetEndEdge());
}

std::vector<unsigned int> Graph::CalculatePathOverLCA(VertexPtr start, VertexPtr end)
{
	std::vector<unsigned int> path;
	path.resize(0);

	//Starte einfach auf Root und gehe solange runter bis die ein Vertexelement unterschiedlich ist
	std::vector<unsigned int> startPathToRoot = start->GetPathToRoot();
	std::vector<unsigned int> endPathToRoot = end->GetPathToRoot();
	int m = startPathToRoot.size()-1;
	int n = endPathToRoot.size()-1;
	while(m >= 0 && n >= 0 && startPathToRoot[m] == endPathToRoot[n]){	m--;	n--;	}
	m++; n++;

	//Erzeuge Path, wobei man mit einem bis zum LCA geht und vom anderem vom LCA zum Ende
	for(int i = 0; i < m; i++) path.push_back(startPathToRoot[i]);
	for(int i = n; i >= 0; i--) path.push_back(endPathToRoot[i]);

	return path;
}

std::vector<VertexPtr> Graph::GetAllVerticesWithoutChild()
{
	std::vector<VertexPtr> verticesWithoutChilds;
	
	//Erzeuge vector mit allen Vertices, welchen keinen Child haben.
	for(int i = 0; i < GetNumberOfVertices(); i++)	
	{
		VertexPtr actVertex = GetVertexWithID(i);
		if(actVertex->GetNumberChildVertices() == 0)
		{
			verticesWithoutChilds.push_back(actVertex);
		}
	}

	return verticesWithoutChilds;
}