#include <QtGui>
#include "MainWindow.h"
#include <QFileDialog>
#include <QRgb>

#include <iostream>
using std::cout;
using std::endl;

MainWindow::MainWindow(QWidget* parent) : QMainWindow(parent)
{
	beta = 0.5f;

	setupUi(this);

	connect(actionOpen, SIGNAL(triggered()),this, SLOT(openFileDialog()));
	connect(actionSave, SIGNAL(triggered()),this, SLOT(saveFileDialog()));
	connect(actionSteuerung, SIGNAL(triggered()),this, SLOT(controlDialog()));
	connect(actionAbout, SIGNAL(triggered()),this, SLOT(aboutDialog()));
	connect( this, SIGNAL(setView(View**)), openGLWidget, SLOT(setView(View**)));
	connect( cb_color, SIGNAL(activated(const QString )), this, SLOT(SphereColorChanged(const QString)));
	connect( cb_hierarchyColor, SIGNAL(activated(const QString)), this, SLOT(HierarchyColorChanged(const QString)));
	connect( cb_relationStartColor, SIGNAL(activated(const QString)), this, SLOT(RelationStartColorChanged(const QString)));
	connect( cb_relationEndColor, SIGNAL(activated(const QString)), this, SLOT(RelationEndColorChanged(const QString)));
	connect( cb_selectionColor, SIGNAL(activated(const QString)), this, SLOT(SelectionColorChanged(const QString)));
}

MainWindow::~MainWindow()
{
	
}

void MainWindow::Mode3D(bool value)
{
	RenderStates::mode3d = value;
}

void MainWindow::SetViewToRooted(bool value)
{
	std::cout << "SET VIEW TO SetViewToRooted" << std::endl;

	if(value)
	{
		RenderStates::actView = View::VM_ROOTEDTREE;
	}
}

void MainWindow::SetViewToBallon(bool value)
{
	std::cout << "SET VIEW TO BALLON" << std::endl;
	if(value)
	{
		RenderStates::actView = View::VM_BALLONTREE;
	}
}

void MainWindow::SetViewToTreeMap(bool value)
{
	if(value)
	{
		if(RenderStates::treemaptype == 0)	RenderStates::actView = View::VM_TREEMAP_SLICE_AND_DICE;
		else								RenderStates::actView = View::VM_TREEMAP_STRIP;
	}
}

void MainWindow::DrawRadialTree(bool value)
{
	if(value)
	{
		RenderStates::actView = View::VM_RADIALTREE;
	}
}

void MainWindow::EnableBlending(bool value)
{
	if(value)	RenderStates::blendingon = true;
	else		RenderStates::blendingon = false;

}

void MainWindow::SetStandardBlending(bool value)
{
	if(value)
	{
		RenderStates::blendingMode = Renderer::STD_BLENDING;
	}
}

void MainWindow::SetMinMaxBlending(bool value)
{
	if(value)
	{
		RenderStates::blendingMode = Renderer::MAX_BLENDING;
	}
}

void MainWindow::SetStandardMinMaxBlending(bool value)
{
	if(value)
	{
		RenderStates::blendingMode = Renderer::STDMAX_BLENDING;
	}
}

void MainWindow::SetAverageWeightingBlending(bool value)
{
	if(value)
	{
		RenderStates::blendingMode = Renderer::AVGWEIGHTING_BLENDING;
	}
}

void MainWindow::SetSummedAverageBlending(bool value)
{
	if(value)
	{
		RenderStates::blendingMode = Renderer::SUMAVG_BLENDING;
	}
}	


void MainWindow::EnableLight(bool value)
{
	RenderStates::lighton = value;
}

void MainWindow::EnableSpotLight(bool value)
{
	if(value)
	{
		RenderStates::lighttype = Light::LT_SPOTLIGHT;
	}
}

void MainWindow::EnableConicLight(bool value)
{
	if(value)
	{
		RenderStates::lighttype = Light::LT_CONIC;
	}
}

void MainWindow::EnablePointLight(bool value)
{
	if(value)
	{
		RenderStates::lighttype = Light::LT_POINT;
	}
}


void MainWindow::EnableShadow(bool value)
{
	RenderStates::enable_shadow = value;
}

void MainWindow::SetVertexInShadow(bool value)
{
	if(value)
	{
		RenderStates::shadow += 1;
	}
	else
	{
		RenderStates::shadow -= 1;
	}	

	std::cout << "RenderStates Shadow: " << RenderStates::shadow << std::endl;
}

void MainWindow::SetHierarchyEdgeInShadow(bool value)
{
	if(value)
	{
		RenderStates::shadow += 2;
	}
	else
	{
		RenderStates::shadow -= 2;
	}

	std::cout << "RenderStates Shadow: " << RenderStates::shadow << std::endl;
}

void MainWindow::SetRelationEdgeInShadow(bool value)
{
	if(value)
	{
		RenderStates::shadow += 4;
	}
	else
	{
		RenderStates::shadow -= 4;
	}	

	std::cout << "RenderStates Shadow: " << RenderStates::shadow << std::endl;
}

void MainWindow::SetGridInShadow(bool value)
{
	if(value)
	{
		RenderStates::shadow += 8;
	}
	else
	{
		RenderStates::shadow -= 8;
	}	

	std::cout << "RenderStates Shadow: " << RenderStates::shadow << std::endl;
}

void MainWindow::DrawNodes(bool value)
{
	if(value)	RenderStates::drawNodes += 1;
	else		RenderStates::drawNodes -= 1;
}

void MainWindow::DrawHierarchyEdges(bool value)
{
	if(value)	RenderStates::drawNodes += 2;
	else		RenderStates::drawNodes -= 2;
}

void MainWindow::DrawRelationEdges(bool value)
{
	if(value)	RenderStates::drawNodes += 4;
	else		RenderStates::drawNodes -= 4;
}

void MainWindow::DrawGrid(bool value)
{
	if(value)	RenderStates::drawNodes += 8;
	else		RenderStates::drawNodes -= 8;

	std::cout << "DrawGrid: " << RenderStates::drawNodes << std::endl;
}

void MainWindow::RotateX(int value)
{
	RenderStates::RotateX = value;
}

void MainWindow::RotateY(int value)
{
	RenderStates::RotateY = value;
}

void MainWindow::RotateZ(int value)
{
	RenderStates::RotateZ = value;
}

void MainWindow::Scale(int value)
{
	RenderStates::Scale = value;
}

void MainWindow::SetLightAttenuate(int value)
{
	RenderStates::LightAttenuate = value;
}

void MainWindow::SetLightRadius(int value)
{
	RenderStates::LightRadius = value;
}

void MainWindow::DrawSliceAndDice(bool value)
{
	if(value) 
	{
		RenderStates::treemaptype = 0;
		RenderStates::actView = View::VM_TREEMAP_SLICE_AND_DICE;
	}
}

void MainWindow::DrawStrips(bool value)
{
	if(value) 
	{
		RenderStates::treemaptype = 1;
		RenderStates::actView = View::VM_TREEMAP_STRIP;
	}
}

void MainWindow::DrawCushionTexture(bool value)
{
	RenderStates::cushionTexture = value;
}

void MainWindow::MirrorRelationEdges(bool value)
{
	RenderStates::mirrorRelationEdges = value;
}

void MainWindow::SphereColorChanged(QString value)
{
	QColor color(value);
	int r, g, b, a;
	color.getRgb(&r,&g,&b,&a);
	RenderStates::vertexColor = vec3((float)(r)/255.f,(float)(g)/255.f,(float)(b)/255.f);
	RenderStates::vertexAlpha = (float)(a)/255.f;
}

void MainWindow::HierarchyColorChanged(QString value)
{
	QColor color(value);
	int r=0, g=0, b=0, a=0;
	color.getRgb(&r,&g,&b,&a);
	RenderStates::hierarchicColor = vec3((float)(r)/255.f,(float)(g)/255.f,(float)(b)/255.f);
	RenderStates::herarchicAlpha = (float)(a)/255.f;

	std::cout << "- hierarchicColor: " << r << " " << g << " " << b << std::endl;
	std::cout << "+ hierarchicColor: " << RenderStates::hierarchicColor.x << " " << RenderStates::hierarchicColor.y << " " << RenderStates::hierarchicColor.z << std::endl;
}

void MainWindow::RelationStartColorChanged(QString value)
{
	QColor color(value);
	int r, g, b, a;
	color.getRgb(&r,&g,&b,&a);
	RenderStates::startRelationColor = vec3((float)(r)/255.f,(float)(g)/255.f,(float)(b)/255.f);
	RenderStates::startRelationAlpha = RenderStates::endRelationAlpha;
}

void MainWindow::RelationEndColorChanged(QString value)
{
	QColor color(value);
	int r, g, b, a;
	color.getRgb(&r,&g,&b,&a);
	RenderStates::endRelationColor = vec3((float)(r)/255.f,(float)(g)/255.f,(float)(b)/255.f);
	RenderStates::endRelationAlpha = RenderStates::endRelationAlpha;
}

void MainWindow::RelationAlpha(int value)
{
	RenderStates::startRelationAlpha = (float)(value)/100.f;
	RenderStates::endRelationAlpha = (float)(value)/100.f;
}

void MainWindow::SelectionColorChanged(const QString value)
{
	QColor color(value);
	int r, g, b, a;
	color.getRgb(&r,&g,&b,&a);
	RenderStates::selectionColor = vec3((float)(r)/255.f,(float)(g)/255.f,(float)(b)/255.f);
	RenderStates::selectionAlpha = (float)(a)/255.f;
}

void MainWindow::openFileDialog()
{
	//Gut: Die Fileloader fr rsf und xml sind komplett gleich aufgebaut. 
	//Es wird in ein gemeinsames Format umgewandelt und darin wird es eingelesen.
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),
                                                 "",
                                                 tr("DataFiles (*.rsf *.xml)"));
	
	//testen ob eine datei ausgewhlt wurde
	if (fileName != QString::null) 
	{
		if(fileName.contains(".rsf", Qt::CaseSensitive))
		{
			//cout<< "es ist .rsf"<< endl;
			auto_ptr<ReadRTFFile> r(new ReadRTFFile());
			std::vector<DataFromFile> datafromfile = r->Read(fileName.toLatin1().data());
			graph = r->ConvertToDataStructure(datafromfile);
			graph->Calculate();

		}else
		{
			//cout<<"kein rsf" <<endl;
			auto_ptr<LoadXMLTree> r(new LoadXMLTree());
			std::vector<DataFromFile> datafromfile = r->Read(fileName.toLatin1().data());
			graph = r->ConvertToDataStructure(datafromfile);
			graph->Calculate();
		}
		View::SetGraph(graph);

	std::cout << "calculate tree" << std::endl;
	view[0] = new RootedTree();
	view[0]->Calculate();
	std::cout << "rootedtree calculated" << std::endl;
	view[1] = new BallonTree();
	view[1]->Calculate();
	std::cout << "ballontree calculated" << std::endl;
	view[2] = new TreeMap(0);
	view[2]->Calculate();	
	std::cout << "slice and dice calculated" << std::endl;
	view[3] = new TreeMap(1);
	view[3]->Calculate();	
	std::cout << "treemap calculated" << std::endl;
	view[4] = new RadialTree();
	view[4]->Calculate();
	std::cout << "radialtree calculated" << std::endl;

		emit setView(view);	

	}
}

void MainWindow::SetBeta(int value)
{
	beta = (float)(value)/1000.f;
}

void MainWindow::BetaChanged()
{
	RenderStates::beta = beta;
}

void MainWindow::saveFileDialog()
{
	QString saveFileName = QFileDialog::getSaveFileName(this, tr("Save Image File"), "", tr("ImageFiles (*.png)"));
	glReadBuffer(GL_FRONT);
	QImage image = openGLWidget->grabFrameBuffer();
	bool saved = image.save(saveFileName);
	if (saved == false) {
		QMessageBox::critical(this,"Error Message","Could not save Image");
	} else {
		QMessageBox::information(this, "Information Message", "Screenshot saved " + saveFileName);
	}
}

void MainWindow::aboutDialog()
{
	QMessageBox::about(this, "Hierarchical Edge Bundle", "Program to display a hierarchical Graph with adjacency Edges \n"
		"by Michael Beham & Caroline Steiner\n" 
		"Informationsvisualisierung, SS 2011\n"
		"Technische Universitt Wien");
}

void MainWindow::controlDialog()
{
	QMessageBox::about(this, "Control Possibilities", " In Addition to the interaction with the User Interface \n you can also interact with the following Keys \n"
		" press ALT right Mousebutton : zoom the whole Scene \n"
		" press ALT left Mousebutton :  rotate the whole Scene\n"		
		" press ALT +  W A S D : position the Ligth Source, marked by the yellow dot \n"	
		" press ALT + R F : the altitude of the Light Source \n"
		);
}