#include "OpenGLWidget.h"
#include <iostream>
#include "RenderStates.h"
#include "glm/glm.hpp"
#include <glm/gtc/matrix_transform.hpp> 
using std::cout;
using std::endl;
using std::cerr;

// global variables
//ggf anpassen!!!
float nearplane = 0.1f;
float farplane = 100.0f;
//field of view angle, in degrees, in the y direction
float fovAngle = 60.0f;

//QGLFormat* OpenGLWidget::format = new QGLFormat();
//QGLContext* OpenGLWidget::context = new QGLContext(*format);

OpenGLWidget::OpenGLWidget(QWidget* parent) : QGLWidget(parent), m_view(0), perspective(false), camera_uvt(0,0,0), translateZ(1)
{
	this->black = QColor::fromRgb(0, 0, 0);
	camera_orth = new Camera(-1.0f, 1.0f, -768.f/1024.f, 768.f/1024.f, 0.0f, 50.0f);
	camera_pers = new Camera(1.0f, 20.0f, 80.0f, 4.0f/3.0f);

	renderer = new RenderImplementation();

	setFocus();

	m_light = new Light();
}

OpenGLWidget::~OpenGLWidget()
{
	delete camera_orth;
	delete camera_pers;
}

void OpenGLWidget::setCameraProjection()
{	
	/*if (perspective) {
		camera->perspective(nearplane, farplane, fovAngle, (float(screenWidth)/float(screenHeight)));
	} else {
		camera->orthogonal((float)-(screenWidth/2), (float)(screenWidth/2), (float)-(screenHeight/2), (float)(screenHeight/2), nearplane, farplane);
	}*/
}

void OpenGLWidget::setView(View** view)
{
	renderer->CreateObjects(view);
}

void OpenGLWidget::initializeGL()
{
	/* if (context()->isValid()==true) {
		cout << "- OpenGL Context is Valid!" << endl;
	} */

	// --- Init GLEW ---
	cout << "- Initializing GLEW..." << endl;
	GLenum err = glewInit(); // initialize GLEW
	if (err != GLEW_OK) {// check for error
		cerr << "- GLEW Error: " << glewGetErrorString(err) << endl;
		exit(1);
	} else {
		cout << "- GLEW initialized!" << endl;
	}

	//Test OpenGL 3.2
	cout << "- Testing OpenGL Version..." << endl;
	if (glewIsSupported("GL_VERSION_3_2")) {
		// retrieve our OpenGL-version
		cout << "- OpenGL Version: " << glGetString(GL_VERSION) << endl;
	} else {
		cout << "- OpenGL 3.2 ist not supported!" << endl;
		cout << "- OpenGL Version: " << glGetString(GL_VERSION) << endl;
		//exit(1);
	}

	// --- DEVIL Loading ---
	ilInit();	
	iluInit();

	// --- Do some setup for OpenGL ---
	// enable back-face culling
	glEnable(GL_CULL_FACE);

	// enable depth-buffer
    glEnable(GL_DEPTH_TEST);
	//glDepthFunc(GL_LESS);//QUAL);
	glDepthFunc(GL_LEQUAL);

	//set clear color
	qglClearColor(black);

	//set Viewport is done in resizeGL

	// --- init matrices ---
	// move camera 4 units backward, i.e. move the scene 4 units forward
	camera_orth->lookAt(glm::vec3(0, -1, 0), glm::vec3(0, 0, 1), glm::vec3(0, 0, 0));
	camera_pers->lookAt(glm::vec3(0, 0, -1), glm::vec3(0, 1, 0), glm::vec3(0, 0, 0));
    
	// initialize the model matrix to identity
	this->model = glm::mat4(1.0f);

	renderer->camera = camera_pers;
	renderer->Initialize();

	// --- done with initializing stuff ---
	glShadeModel(GL_SMOOTH);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glLineWidth(1.0);
	glEnable(GL_LINE_SMOOTH);
	glClearDepth(1.0f);
}

void OpenGLWidget::resizeGL(int width, int height)
{
	if (height == 0) height = 1;
	glViewport(0, 0, width, height);
	this->screenWidth = width;
	this->screenHeight = height; 

	//this->setCameraProjection();
	//updateGL();
}

void OpenGLWidget::paintGL()
{
	Camera* camera;
	mode3D = RenderStates::mode3d;
	if(mode3D)	camera = camera_pers;
	else		camera = camera_orth;

	QTime timer;
	timer.start();

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	renderer->DrawObjects(camera, m_light);
	renderer->DrawHUD(timer.elapsed());
	timer.restart();
	update();
}

void OpenGLWidget::mousePressEvent(QMouseEvent *event)
{ 
	setFocus();
	std::cout << " mousePressEvent " << std::endl;	
	QPoint lastPos = event->pos();
	m_oldMousePosition = vec3(lastPos.x(), lastPos.y(), 0.0);
}	

void OpenGLWidget::mouseMoveEvent(QMouseEvent *event)
{
	std::cout << " mouseMoveEvent " << std::endl;
	QPoint lastPos = event->pos();
	vec3 move;

	if(event->modifiers() == Qt::AltModifier)
	{
		if (event->buttons() == Qt::RightButton) 
		{
			move.x = ((lastPos.x()-m_oldMousePosition.x)/512.f);
			move.y = ((lastPos.y()-m_oldMousePosition.y)/384.f);
			move.z = 0.0;
			m_oldMousePosition = vec3(lastPos.x(), lastPos.y(), 0.0);		
			if(mode3D)	
			{				
				vec3 move2 = vec3(0,0,sqrt((move.x*move.x)+(move.y*move.y)));
				if(move.x + move.y < 0.f) 	move2 *= -1.f;
				move2*=2.0;

				translateZ += move2.z;

				mat4 rot;
				mat4 erg = glm::gtc::matrix_transform::rotate(rot , camera_uvt.x*180.f, vec3(0,1,0));	
				mat4 erg2 = glm::gtc::matrix_transform::rotate(rot , camera_uvt.y*90.f, vec3(1,0,0));	
				mat4 erg3 = glm::gtc::matrix_transform::translate(rot , vec3(0,0,translateZ));	
				vec4 pos = erg * erg2 * erg3 * vec4(0,0,0,1);

				camera_pers->lookAt(vec3(pos.x,pos.y, pos.z), vec3(0,1,0), vec3(0,0,0));
			}
			else
			{
				vec3 move2 = vec3(0,0,sqrt((move.x*move.x)+(move.y*move.y)));
				if(move.x + move.y < 0.f) 	move2 *= -1.f;
				move2*=2.0;

				camera_orth->setScreenWidth(camera_orth->getScreenWidth()+move2.z);
				camera_orth->setScreenHeight(camera_orth->getScreenHeight()+move2.z);
				std::cout << "m_oldMousePosition.x: " << camera_orth->getScreenWidth() << " m_oldMousePosition.y: "<< camera_orth->getScreenHeight() << std::endl;
				camera_orth->updateOrthogonalView();
			}
		}
	
		if (event->buttons() == Qt::LeftButton) 
		{
			move.x = ((lastPos.x()-m_oldMousePosition.x)/512.0f);
			move.y = ((lastPos.y()-m_oldMousePosition.y)/384.0f);
	
			m_oldMousePosition = vec3(lastPos.x(), lastPos.y(), 0.0);
	
			if(mode3D)	
			{
				vec4 toEyePoint(-camera_pers->getPosition().x,-camera_pers->getPosition().y,-camera_pers->getPosition().z, 1.0f); //Look to vec3(0,0,0);
				camera_uvt.x += move.x;
				camera_uvt.y += move.y;

				mat4 rot;
				mat4 erg = glm::gtc::matrix_transform::rotate(rot , camera_uvt.x*180.f, vec3(0,1,0));	
				mat4 erg2 = glm::gtc::matrix_transform::rotate(rot , camera_uvt.y*90.f, vec3(1,0,0));	
				mat4 erg3 = glm::gtc::matrix_transform::translate(rot , vec3(0,0,translateZ));	

				vec4 pos = erg * erg2 * erg3 * vec4(0,0,0,1);

				camera_pers->lookAt(vec3(pos.x,pos.y, pos.z), vec3(0,1,0), vec3(0,0,0));
			}
			else
			{
				move.x *= -1.0f;
				vec3 pos = camera_orth->getPosition() + vec3(move.x,move.z,move.y);
				camera_orth->lookAt(pos, vec3(0,0,1), vec3(pos.x,0,pos.z));
			}
		}
	}
	else		renderer->Picking(lastPos.x(), lastPos.y());
}

void OpenGLWidget::mouseReleaseEvent(QMouseEvent * event)
{	
	std::cout << " Stop Picking" << std::endl;
	std::cout << " mouseReleaseEvent " << std::endl;
}

void OpenGLWidget::keyPressEvent(QKeyEvent *event)
{
	if(event->key() == Qt::Key_W && event->modifiers() == Qt::AltModifier)
	{		
		m_light->SetPosition(m_light->GetPosition() + vec3(0,0,0.05));
	}
	if(event->key() == Qt::Key_A && event->modifiers() == Qt::AltModifier )
	{
		m_light->SetPosition(m_light->GetPosition() + vec3(0.05,0,0));
	}
	if(event->key() == Qt::Key_D && event->modifiers() == Qt::AltModifier )
	{
		m_light->SetPosition(m_light->GetPosition() - vec3(0.05,0,0));
	}
	if(event->key() == Qt::Key_S && event->modifiers() == Qt::AltModifier )
	{
		m_light->SetPosition(m_light->GetPosition() - vec3(0,0,0.05));
	}
	if(event->key() == Qt::Key_R && event->modifiers() == Qt::AltModifier )
	{
		m_light->SetPosition(m_light->GetPosition() + vec3(0,0.05,0));
	}
	if(event->key() == Qt::Key_F && event->modifiers() == Qt::AltModifier )
	{
		m_light->SetPosition(m_light->GetPosition() - vec3(0,0.05,0));
	}

	if(event->key() == Qt::Key_T )
	{
		//renderer->UpdateVBOTest();
	}
}

void OpenGLWidget::keyReleaseEvent(QKeyEvent *event)
{

}