#include "RenderImplementation.h"

RenderImplementation::RenderImplementation(void)
{
	m_view = 0;
	for(int i  = 0; i < View::VM_QUANTITY; i++) tree[i] = 0;
	selectedObj = new SelectedObject();
	splineSet = NULL;
	m_beta = 0.5f;
}

RenderImplementation::~RenderImplementation(void)
{
	delete selectedObj;
	if(splineSet) delete splineSet;
}

void RenderImplementation::Initialize()
{
	std::cout << "Initialize" << std::endl;

	glClearColor(0,0,0,1);

	/////////////////////////////////////////////
	// --- init Shader ---
	// lighting and shadow shader 
	shaderobj = new ShaderObj();
	shaderobj->vertexshader = "shader/shading_dir_vert.glsl";
	shaderobj->fragmentshader = "shader/shading_dir_frag.glsl";
	shaderobj->parameters.push_back(new ParameterObj());
	shaderobj->parameters[shaderobj->parameters.size()-1]->type = ParameterObj::PT_INT;
	shaderobj->parameters[shaderobj->parameters.size()-1]->var = "ShadowMap";
	shaderobj->parameters.push_back(new ParameterObj());
	shaderobj->parameters[shaderobj->parameters.size()-1]->type = ParameterObj::PT_INT;
	shaderobj->parameters[shaderobj->parameters.size()-1]->var = "enable_shadow";
	shaderobj->parameters.push_back(new ParameterObj());
	shaderobj->parameters[shaderobj->parameters.size()-1]->type = ParameterObj::PT_INT;
	shaderobj->parameters[shaderobj->parameters.size()-1]->var = "object_type";
	shaderobj->parameters.push_back(new ParameterObj());
	shaderobj->parameters[shaderobj->parameters.size()-1]->type = ParameterObj::PT_INT;
	shaderobj->parameters[shaderobj->parameters.size()-1]->var = "light_type";
	shaderobj->parameters.push_back(new ParameterObj());
	shaderobj->parameters[shaderobj->parameters.size()-1]->type = ParameterObj::PT_INT;
	shaderobj->parameters[shaderobj->parameters.size()-1]->var = "Texture0";
	shaderobj->parameters.push_back(new ParameterObj());
	shaderobj->parameters[shaderobj->parameters.size()-1]->type = ParameterObj::PT_FLOAT4;
	shaderobj->parameters[shaderobj->parameters.size()-1]->var = "uColor";
	/*shaderobj->parameters.push_back(new ParameterObj());
	shaderobj->parameters[shaderobj->parameters.size()-1]->type = ParameterObj::PT_MAT4;
	shaderobj->parameters[shaderobj->parameters.size()-1]->var = "invModelViewMatrix";*/
	lightningshader = new Shader(shaderobj);

	shaderobj_shadow = new ShaderObj();
	shaderobj_shadow->vertexshader = "shader/shadow_vert.glsl";
	shaderobj_shadow->fragmentshader = "shader/shadow_frag.glsl";
	shadowshader = new Shader(shaderobj_shadow);

	// Transpancy shader
	// Transpancy shader
	shaderobj_sum = new ShaderObj();
	shaderobj_sum->vertexshader = "shader/sum_vert.glsl";
	shaderobj_sum->fragmentshader = "shader/sum_frag.glsl";
	shaderobj_sum->parameters.push_back(new ParameterObj());
	shaderobj_sum->parameters[shaderobj_sum->parameters.size()-1]->type = ParameterObj::PT_INT;
	shaderobj_sum->parameters[shaderobj_sum->parameters.size()-1]->var = "light_type";
	initsumshader = new Shader(shaderobj_sum);

	shaderobj_finalsum = new ShaderObj();
	shaderobj_finalsum->vertexshader = "shader/sum_final_vert.glsl";
	shaderobj_finalsum->fragmentshader = "shader/sum_final_frag.glsl";
	shaderobj_finalsum->parameters.push_back(new ParameterObj());
	shaderobj_finalsum->parameters[shaderobj_finalsum->parameters.size()-1]->type = ParameterObj::PT_INT;
	shaderobj_finalsum->parameters[shaderobj_finalsum->parameters.size()-1]->var = "ColorTex0";
	shaderobj_finalsum->parameters.push_back(new ParameterObj());
	shaderobj_finalsum->parameters[shaderobj_finalsum->parameters.size()-1]->type = ParameterObj::PT_INT;
	shaderobj_finalsum->parameters[shaderobj_finalsum->parameters.size()-1]->var = "Background";
	finalsumshader = new Shader(shaderobj_finalsum);

	shaderobj_weighting = new ShaderObj();
	shaderobj_weighting->vertexshader = "shader/weighting_vert.glsl";
	shaderobj_weighting->fragmentshader = "shader/weighting_frag.glsl";
	shaderobj_weighting->parameters.push_back(new ParameterObj());
	shaderobj_weighting->parameters[shaderobj_weighting->parameters.size()-1]->type = ParameterObj::PT_INT;
	shaderobj_weighting->parameters[shaderobj_weighting->parameters.size()-1]->var = "light_type";
	initweightingshader = new Shader(shaderobj_weighting);

	shaderobj_blending = new ShaderObj();
	shaderobj_blending->vertexshader = "shader/relation_occlusion_vert.glsl";
	shaderobj_blending->fragmentshader = "shader/relation_occlusion_frag.glsl";
	shaderobj_blending->parameters.push_back(new ParameterObj());
	shaderobj_blending->parameters[shaderobj_blending->parameters.size()-1]->type = ParameterObj::PT_INT;
	shaderobj_blending->parameters[shaderobj_blending->parameters.size()-1]->var = "DepthBackground";
	initblendingshader = new Shader(shaderobj_blending);

	shaderobj_finalweighting = new ShaderObj();
	shaderobj_finalweighting->vertexshader = "shader/weighting_final_vert.glsl";
	shaderobj_finalweighting->fragmentshader = "shader/weighting_final_frag.glsl";
	shaderobj_finalweighting->parameters.push_back(new ParameterObj());
	shaderobj_finalweighting->parameters[shaderobj_finalweighting->parameters.size()-1]->type = ParameterObj::PT_INT;
	shaderobj_finalweighting->parameters[shaderobj_finalweighting->parameters.size()-1]->var = "ColorTex0";
	shaderobj_finalweighting->parameters.push_back(new ParameterObj());
	shaderobj_finalweighting->parameters[shaderobj_finalweighting->parameters.size()-1]->type = ParameterObj::PT_INT;
	shaderobj_finalweighting->parameters[shaderobj_finalweighting->parameters.size()-1]->var = "ColorTex1";
	shaderobj_finalweighting->parameters.push_back(new ParameterObj());
	shaderobj_finalweighting->parameters[shaderobj_finalweighting->parameters.size()-1]->type = ParameterObj::PT_FLOAT3;
	shaderobj_finalweighting->parameters[shaderobj_finalweighting->parameters.size()-1]->var = "Background";
	finalweightingshader = new Shader(shaderobj_finalweighting);
	/////////////////////////////////////////////

	// --- init FBO ---
	fbo_opacity = new FBO(1024,768,2);
	fbo_transparency = new FBO(1024,768,2);
	fbo_transparency2 = new FBO(1024,768,1);
	fbo_shadow = new FBO(1024,768,1);

	//--- Display Lists ----
	RenderPrimitives::Initialize();
	
	//---Load Textures ---
	RenderTextures::Initialize();

	//---- Load Font ---
	LoadFont();
	
	//Problem: GLU 1.2 kann nur Viewdependent NURBS zeichnen. Daher auch mit Displaylists total langsam. GLU 1.3 knnte das beheben, aber Win hat nur 1.2 ^^
// 	theNurb = gluNewNurbsRenderer();
// 	glEnable(GL_MAP1_COLOR_4);

	cam_shadow = new Camera(-2.0, 2.0f, -2.f*768.f/1024.f, 2.f*768.f/1024.f, -50.0f, 50.0f);
	//UpdateLight();
	
	//glEnable(GL_LIGHTING);
	//---- Das muss unbedingt fr jeden Treetype gesetzt weden
	tree[0] = new RenderRootedTree();
	tree[1] = new RenderBallonTree();
	tree[2] = new RenderTreeMap();
	tree[3] = new RenderTreeMap();
	tree[4] = new RenderRadialTree();
	//--------------------------------------------------------

	RenderStates::actView = View::VM_BALLONTREE;
	RenderStates::enable_shadow = true;
	//UpdateStates();
	std::cout << "Initialized" << std::endl;

	sphereColor = vec3(1.0,0.0,0.0);
	sphereAlpha=1.f;
}

void RenderImplementation::UpdateLight()
{
	float ambient[] =  {0.1f, 0.1f, 0.1f, 1.0f};
	float diffuse[] =  {0.8f, 0.8f, 0.8f, 1.0f};
	float specular[] = {1.0f, 1.0f, 1.0f, 1.0f};
	float position[] = {m_light->GetPosition().x, m_light->GetPosition().y, m_light->GetPosition().z, 1.0f};
	float spotdirection[] = { m_light->GetSpotDirection().x, m_light->GetSpotDirection().y, m_light->GetSpotDirection().z};

	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR,specular);
	glLightfv(GL_LIGHT0, GL_POSITION, position);

	glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, spotdirection);
	glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, 0.0f);
	if(m_light->GetType() == Light::LT_CONIC)	
	{
		glLightf(GL_LIGHT0, GL_SPOT_CUTOFF,	m_light->GetRadius()/100.f);
		glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, (m_light->GetRadius()+(m_light->GetAttentuate()*2.f))/100.f);
	}
	else
	{
		glLightf(GL_LIGHT0, GL_SPOT_CUTOFF,	m_light->GetRadius()/100.f);
		glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, (m_light->GetRadius()+m_light->GetAttentuate())/100.f);
	}

	glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, m_light->GetAttentuate()*0.25f);
	glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, m_light->GetAttentuate()*0.125f);

	//UP VEKTOR NOCH BESTIMMEN!!!!
	//cam_shadow = new Camera(0.1f, 6.0f ,40.0f, 4.0f/3.0f);
	cam_shadow->lookAt(m_light->GetPosition(), vec3(0,0,1), m_light->GetPosition()+m_light->GetSpotDirection());
}

void RenderImplementation::CreateObjects(View** view)
{
	std::cout << "Create Objects" << std::endl;
	
	m_view = view;
	for(int i = 0; i < View::VM_QUANTITY; i++)	tree[i]->m_view = m_view[i];

	if(m_view != 0)
	{
		if(splineSet) delete splineSet;
		numberofHierarchies = m_view[actView]->m_pkGraph->GetNumberOfHierarchies();
		splineSet = new SplineSet(numberofHierarchies);

		int tmpView = actView;
		for(actView = 0; actView < View::VM_QUANTITY; actView++)
		{
			numberofhierarchyedges = m_view[actView]->m_pkGraph->GetNumberOfHiearchicEdges();
			vec3* hieararchylines = new vec3[numberofhierarchyedges*2.0];
			vec4* hieararchycolor = new vec4[numberofhierarchyedges*2.0];
			vec4* hieararchypicking = new vec4[numberofhierarchyedges*2.0];

			unsigned int n = 0;
			for(unsigned int i = 0; i < numberofhierarchyedges; i++)
			{			
				hieararchylines[n] = glm::vec3(m_view[actView]->GetGUIVertexForView(m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetStartEdge()->GetID()).GetPosition().x,1.0f-m_view[actView]->GetGUIVertexForView(m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetStartEdge()->GetID()).GetPosition().y,	m_view[actView]->GetGUIVertexForView(m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetStartEdge()->GetID()).GetPosition().z);
				hieararchycolor[n] = glm::vec4(m_view[actView]->GetGUIVertexForView(m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetStartEdge()->GetID()).GetColor().x,	m_view[actView]->GetGUIVertexForView(m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetStartEdge()->GetID()).GetColor().y,			m_view[actView]->GetGUIVertexForView(m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetStartEdge()->GetID()).GetColor().z, 1.0f/*/m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetStartEdge()->GetGUIVertexForView(actView).GetOpacity()*/);
				hieararchypicking[n] = glm::vec4(0.0f, (float)(i)/(float)(numberofhierarchyedges), 0.0f,1.0f);
				n++;
				hieararchylines[n] = glm::vec3(m_view[actView]->GetGUIVertexForView(m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetEndEdge()->GetID()).GetPosition().x,	1.0f-m_view[actView]->GetGUIVertexForView(m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetEndEdge()->GetID()).GetPosition().y,	m_view[actView]->GetGUIVertexForView(m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetEndEdge()->GetID()).GetPosition().z);
				hieararchycolor[n] = glm::vec4(m_view[actView]->GetGUIVertexForView(m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetEndEdge()->GetID()).GetColor().x,		m_view[actView]->GetGUIVertexForView(m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetEndEdge()->GetID()).GetColor().y,			m_view[actView]->GetGUIVertexForView(m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetEndEdge()->GetID()).GetColor().z, 1.0f/*/m_view[actView]->m_pkGraph->GetHierarchyEdgeAt(i)->GetEndEdge()->GetGUIVertexForView(actView).GetOpacity()*/);
				hieararchypicking[n] = glm::vec4(0.0f, (float)(i)/(float)(numberofhierarchyedges), 0.0f,1.0f);
				n++;
			}
		
			glEnableClientState( GL_VERTEX_ARRAY );				// Disable Vertex Arrays
			glEnableClientState( GL_COLOR_ARRAY );				// Disable Color Arrays
			glEnableClientState( GL_TEXTURE_COORD_ARRAY );		// Disable Vertex Arrays

			// Generate And Bind The Vertex Buffer
			glGenBuffers( 1, &m_view[actView]->vbohierarchylines_vert );							// Get A Valid Name
			glBindBuffer( GL_ARRAY_BUFFER, m_view[actView]->vbohierarchylines_vert );			// Bind The Buffer
			glBufferData( GL_ARRAY_BUFFER, numberofhierarchyedges*2*sizeof(vec3), hieararchylines, GL_DYNAMIC_DRAW_ARB );	// Load The Data

			glGenBuffers( 1, &m_view[actView]->vbohierarchylines_color );							// Get A Valid Name
			glBindBuffer( GL_ARRAY_BUFFER, m_view[actView]->vbohierarchylines_color );			// Bind The Buffer
			glBufferData( GL_ARRAY_BUFFER, numberofhierarchyedges*2*sizeof(vec4), hieararchycolor, GL_DYNAMIC_DRAW_ARB );	// Load The Data

			glGenBuffersARB( 1, &m_view[actView]->vboindex_picking_ind );							// Get A Valid Nam
			glBindBufferARB( GL_ARRAY_BUFFER_ARB, m_view[actView]->vboindex_picking_ind );			// Bind The Buffere
			glBufferDataARB( GL_ARRAY_BUFFER_ARB, numberofhierarchyedges*2*sizeof(vec4), hieararchypicking, GL_DYNAMIC_DRAW_ARB);
				
			glDisableClientState(GL_VERTEX_ARRAY);
			glDisableClientState(GL_COLOR_ARRAY );				
			glDisableClientState(GL_TEXTURE_COORD_ARRAY );

			delete [] hieararchylines; 
			hieararchylines = 0;
	
			// fr alle Splines

			numreledges = m_view[actView]->m_pkGraph->GetNumberOfRelationEdges();
			QVector<QVector3D>* P = new QVector<QVector3D>();
			P->resize(0);   

			for (int i = 1; i < numreledges; i++)
			{
				P->clear();
				//Zum holen der Informationen 

				RelationPtr r = m_view[actView]->m_pkGraph->GetRelationEdgeAt(i);	//Hole Relationskante
				std::vector<unsigned int> relationids = m_view[actView]->m_pkGraph->CalculatePathOverLCA(r);	//Hole Liste der Vertices ID
				for(unsigned int j = 0; j < relationids.size(); j++)
				{
					vec3 vecGet = m_view[actView]->GetGUIVertexForView(m_view[actView]->m_pkGraph->GetVertexWithID(relationids[j])->GetID()).GetPosition();
					QVector3D vec;
					vec.setX(vecGet.x);

					if(j > 0 && j < relationids.size()-1)
					{
						switch(actView)
						{
						case 0:	vec.setY(1.325f-vecGet.y);
							break;
						case 1:	vec.setY(-vecGet.y);
							break;
						case 2:	
						case 3:	vec.setY(0.5);
							break;
						}
					}
					else vec.setY(vecGet.y);

					vec.setZ(vecGet.z);
					P->push_back(vec);
				}	
				// add Splines
				splineSet->addBSpline(P, relationids.size(), actView, i);
			}
		}
		actView = tmpView;
		splineSet->sortBSpline();
	}
// 	std::cout << "Objects created" << std::endl;
// 	QVector<BSpline*>* vector = splineSet->getSplineVector(actView);
// 	for (int i = 0; i < vector->size(); i++)
// 	{
// 		cout << "Vector at " << i << " laenge: " << vector->at(i)->getLength() << " " << endl;  
// 	}
}

void RenderImplementation::DrawShadowPass(Matrix& lightmatrix)
{
	if(RenderStates::enable_shadow)
	{
		fbo_shadow->Bind();
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		BeginCamera(cam_shadow);
		lightmatrix = lightmatrix * GetProjectionMatrix() * GetModelViewMatrix();
		shadowshader->start(shaderobj_shadow);

		glPushMatrix();
		glTranslatef(0,-2,0);
		glScalef(4,0.5,4);
			RenderPrimitives::DrawCube();
		glPopMatrix();

		if(m_light->AreVerticesInShadow()) DrawAllVertices();
		if(m_light->AreGridsInShadow()) DrawGrid();
		if(m_light->AreHierarchyEdgeInShadow()) DrawHierarchyEdges(false);
		if(m_light->AreRelationEdgeInShadow()) DrawRelationEdges();
		
		shadowshader->stop();
		EndCamera();
		fbo_shadow->Unbind();
	}
}

void RenderImplementation::DrawShadow(Matrix lightmatrix)
{
	//----- Draw Shadows ----
	lightmatrix = lightmatrix * GetModelViewMatrix().GetInverse();
	
	glActiveTexture(GL_TEXTURE7);
	glMatrixMode(GL_TEXTURE);
	glLoadMatrixf(lightmatrix.Get());
	glBindTexture(GL_TEXTURE_2D, fbo_shadow->tex1[0]);
	glEnable(GL_TEXTURE_2D);
	glMatrixMode(GL_MODELVIEW); 

	if(RenderStates::enable_shadow)
	{
		glPushMatrix();
		glTranslatef(0,-2,0);
		glScalef(1,0.5,1);
			RenderPrimitives::DrawCube();
		glPopMatrix();
	}

	glActiveTexture(GL_TEXTURE7);
	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();
	glDisable(GL_TEXTURE_2D);
	glMatrixMode(GL_MODELVIEW);
	glActiveTexture(GL_TEXTURE0);
	//-----------------------
}

void RenderImplementation::DrawResult()
{
	switch(blendingmode)
	{
	case Renderer::STDMAX_BLENDING:	
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		//glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glBlendFunc(GL_ONE, GL_ONE);
		glBlendEquation(GL_FUNC_ADD);
		glEnable(GL_BLEND);
		DrawScreenAlignedQuad(fbo_opacity->tex1[0], 0, 0, 0, 0,0,0);
		DrawScreenAlignedQuad(fbo_transparency2->tex1[0], 0, 0, 0, 0,0,0);
		glDisable(GL_BLEND);	
		break;
	case Renderer::SUMAVG_BLENDING:	
		glUseProgramObjectARB(shaderobj_finalsum->shaderID);
		glUniform1iARB(shaderobj_finalsum->parameters[0]->id, 0);
		glUniform1iARB(shaderobj_finalsum->parameters[1]->id, 1);
		DrawScreenAlignedQuad(fbo_transparency2->tex1[0], fbo_opacity->tex1[0],0, 0, 0,0,0);
		glUseProgramObjectARB(0);
		break;
	case Renderer::AVGWEIGHTING_BLENDING:	
		glUseProgramObjectARB(shaderobj_finalweighting->shaderID);
		glUniform1iARB(shaderobj_finalweighting->parameters[0]->id, 0);
		glUniform1iARB(shaderobj_finalweighting->parameters[1]->id, 1);
		glUniform1iARB(shaderobj_finalweighting->parameters[2]->id, 2);
		DrawScreenAlignedQuad(fbo_transparency->tex1[0], fbo_transparency->tex1[1], fbo_opacity->tex1[0], 0, 0,0,0);
		glUseProgramObjectARB(0);
		break;
	}
}

void RenderImplementation::DrawVerticesPass(Matrix lightmatrix)
{
	fbo_opacity->Bind();	fbo_opacity->DrawBuffers();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	BeginCamera();
	
	Matrix invModelviewmat = GetModelViewMatrix().GetInverse();

	glUseProgramObjectARB(shaderobj->shaderID);
	glUniform1i(shaderobj->parameters[0]->id, 7);
	glUniform1i(shaderobj->parameters[1]->id, 1);
	glUniform1i(shaderobj->parameters[2]->id, 0);
	if(!m_light->IsOn())	glUniform1i(shaderobj->parameters[3]->id, 4);
	else					glUniform1i(shaderobj->parameters[3]->id, m_light->GetType());
	glUniform1i(shaderobj->parameters[4]->id, 0);
	glUniform4f(shaderobj->parameters[5]->id, 1.0f, 1.0f, 1.0f, 1.0f);	//uColor

	glActiveTexture(GL_TEXTURE6);
	glMatrixMode(GL_TEXTURE);
	glLoadMatrixf(invModelviewmat.Get());
	glEnable(GL_TEXTURE_2D);
	glMatrixMode(GL_MODELVIEW);
	glActiveTexture(GL_TEXTURE0);

	glColor4f(1,1,1,1);
	if(m_light->IsShadowOn()) DrawShadow(lightmatrix);

	//Drawing nodes	
	glUniform1i(shaderobj->parameters[1]->id, 0);	//disable shadows
	glUniform1i(shaderobj->parameters[2]->id, 0);	//disable shadows
	DrawVertices();
	
	glUniform1i(shaderobj->parameters[2]->id, 3);	//lighting + spot + texture 
	DrawGrid();
	
	glUniform1i(shaderobj->parameters[2]->id, 1);	//lighting + spot + texture 
	glUniform4f(shaderobj->parameters[5]->id, 1.0f, 1.0f, 1.0f, 1.0f);	//uColor
	glColor4f(1,1,1,1);
	DrawHierarchyEdges();	

	if(blendingmode != Renderer::STDMAX_BLENDING && blendingmode != Renderer::MAX_BLENDING) 
	{
		glUniform1i(shaderobj->parameters[2]->id, 4);
		DrawRelationEdgePassMethod3();
	}

	glUniform1i(shaderobj->parameters[2]->id, 1);	//spot + glColor
	if(blendingmode != Renderer::SUMAVG_BLENDING && blendingmode != Renderer::AVGWEIGHTING_BLENDING)  DrawRelationEdgePassMethod1();
	glColor4f(1,1,1,1);

	glUniform1i(shaderobj->parameters[2]->id, 0);
	glUniform1i(shaderobj->parameters[3]->id, 4);
	glUniform1i(shaderobj->parameters[4]->id, 0);
	glUniform4f(shaderobj->parameters[5]->id, 1.0f, 1.0f, 0.0f, 1.0f);	
	glPushMatrix();
	glTranslatef(m_light->GetPosition().x, m_light->GetPosition().y, m_light->GetPosition().z); 
	RenderPrimitives::DrawSphere();
	glPopMatrix();

	glActiveTexture(GL_TEXTURE6);
	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();
	glDisable(GL_TEXTURE_2D);
	glMatrixMode(GL_MODELVIEW);
	glActiveTexture(GL_TEXTURE0);

	glUseProgramObjectARB(0);
	EndCamera();
	fbo_opacity->Unbind();
}
void RenderImplementation::DrawRelationEdgePassMethod3()
{
	glDepthMask(false);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);

	glBlendEquation(GL_MAX);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	DrawRelationEdges();
 
	glDisable(GL_BLEND);
	glDepthMask(true);
	glEnable(GL_DEPTH_TEST);

	glBlendEquation(GL_FUNC_ADD);
	//##############################
}
void RenderImplementation::DrawRelationEdgePassMethod2()
{
	glClearColor(0,0,0,0);
	
	fbo_transparency2->Bind();
	fbo_transparency2->DrawBuffers();
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
	glDepthMask(true);
	glEnable(GL_DEPTH_TEST);

	//Drawing the Depthbuffer from FBO_OPACITY TO THIS FBO
	glUseProgramObjectARB(shaderobj_blending->shaderID);	
	glUniform1iARB(shaderobj_blending->parameters[0]->id, 0);	
	DrawScreenAlignedQuad(fbo_opacity->depthbuffer1, 0, 0, 0, 0,0,0);
	glUseProgramObjectARB(0);

	BeginCamera();
	Matrix invModelviewmat = GetModelViewMatrix().GetInverse();
	glDepthMask(false);
	glEnable(GL_DEPTH_TEST);

	glBlendEquation(GL_FUNC_ADD);	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	
	glActiveTexture(GL_TEXTURE6);
	glMatrixMode(GL_TEXTURE);
	glLoadMatrixf(invModelviewmat.Get());
	glEnable(GL_TEXTURE_2D);
	glMatrixMode(GL_MODELVIEW);
	glActiveTexture(GL_TEXTURE0);

	DrawRelationEdges();

	glActiveTexture(GL_TEXTURE6);
	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();
	glDisable(GL_TEXTURE_2D);
	glMatrixMode(GL_MODELVIEW);
	glActiveTexture(GL_TEXTURE0);

	glDisable(GL_BLEND);	
	glEnable(GL_DEPTH_TEST);
	glDepthMask(true);
	glEnable(GL_DEPTH_TEST);

	EndCamera();
	fbo_transparency2->Unbind();
}


void RenderImplementation::DrawRelationEdgePassMethod1()
{
	glDepthMask(false);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	
	if(blendingmode == Renderer::STDMAX_BLENDING)
	{
		//This problem can be resolved byadrawing short curves on top of long curves
		glBlendEquation(GL_MAX);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);		
		DrawRelationEdges();
	}
	//This problem can be resolved by drawing short curves on top of long curves

	if(blendingmode == Renderer::MAX_BLENDING) glBlendEquation(GL_MAX);
	else	glBlendEquation(GL_FUNC_ADD);

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	DrawRelationEdges();

	glDisable(GL_BLEND);
	glDepthMask(true);
	glEnable(GL_DEPTH_TEST);

	//glClear(GL_DEPTH_BUFFER_BIT);
	glBlendEquation(GL_FUNC_ADD);
	//##############################
}

void RenderImplementation::DrawRelationEdgePass()
{
	//##############################
	glClearColor(0,0,0,0);
	if(blendingmode == Renderer::AVGWEIGHTING_BLENDING) 
	{
		fbo_transparency->Bind();
		fbo_transparency->DrawBuffers();
	}
	else
	{
		fbo_transparency2->Bind();
		fbo_transparency2->DrawBuffers();
	}
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
	glDepthMask(true);
	glEnable(GL_DEPTH_TEST);

	//Drawing the Depthbuffer from FBO_OPACITY TO THIS FBO
	glUseProgramObjectARB(shaderobj_blending->shaderID);	
	glUniform1iARB(shaderobj_blending->parameters[0]->id, 0);	
	DrawScreenAlignedQuad(fbo_opacity->depthbuffer1, 0, 0, 0, 0,0,0);
	glUseProgramObjectARB(0);

	BeginCamera();
	Matrix invModelviewmat = GetModelViewMatrix().GetInverse();
	glDepthMask(false);
	glEnable(GL_DEPTH_TEST);

	glBlendEquation(GL_FUNC_ADD);
	glBlendFunc(GL_ONE, GL_ONE);
	glEnable(GL_BLEND);

	//Drawing Edges
	if(blendingmode == Renderer::AVGWEIGHTING_BLENDING)	
	{
		glUseProgramObjectARB(shaderobj_weighting->shaderID);	
		if(!m_light->IsOn())	glUniform1i(shaderobj_weighting->parameters[0]->id, 4);
		else					glUniform1i(shaderobj_weighting->parameters[0]->id, m_light->GetType());

	}
	if(blendingmode == Renderer::SUMAVG_BLENDING)	
	{
		glClearColor(0,0,0,0);
		glClear(GL_COLOR_BUFFER_BIT);
		glClearColor(0,0,0,0);
		glUseProgramObjectARB(shaderobj_sum->shaderID);
		if(!m_light->IsOn())	glUniform1i(shaderobj_sum->parameters[0]->id, 4);
		else					glUniform1i(shaderobj_sum->parameters[0]->id, m_light->GetType());	
	}

	glActiveTexture(GL_TEXTURE6);
	glMatrixMode(GL_TEXTURE);
	glLoadMatrixf(invModelviewmat.Get());
	glEnable(GL_TEXTURE_2D);
	glMatrixMode(GL_MODELVIEW);
	glActiveTexture(GL_TEXTURE0);

	DrawRelationEdges();

	glActiveTexture(GL_TEXTURE6);
	glMatrixMode(GL_TEXTURE);
	glLoadIdentity();
	glDisable(GL_TEXTURE_2D);
	glMatrixMode(GL_MODELVIEW);
	glActiveTexture(GL_TEXTURE0);

	glUseProgramObjectARB(0);

	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);

	glDepthMask(true);
	glEnable(GL_DEPTH_TEST);

	EndCamera();
	glClear(GL_DEPTH_BUFFER_BIT);
	if(blendingmode == Renderer::AVGWEIGHTING_BLENDING)  fbo_transparency->Unbind();
	else	fbo_transparency2->Unbind();
}

void RenderImplementation::DrawObjects(Camera* camera, Light* light)
{
	this->camera = camera;
	this->m_light = light;
	UpdateStates();
	UpdateLight();

	Matrix lightmatrix(0.5,0,0,0.5, 0,0.5,0,0.5, 0,0,0.5,0.5, 0.0,0.0,0.0,1);

	glViewport(0, 0, 1024, 768);		//FBO Resize sehr langsam, lieber eine gre festlegen und dann immer hinein rendern (daher viewport auf 1024,768) gesetzt.
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	if(m_light->IsShadowOn())	DrawShadowPass(lightmatrix);
	DrawVerticesPass(lightmatrix);

	if(blendingmode == Renderer::SUMAVG_BLENDING || blendingmode == Renderer::AVGWEIGHTING_BLENDING) 
	{
		DrawRelationEdgePass();
		DrawResult();
	}
	else
	{
		if(blendingmode == Renderer::SUMAVG_BLENDING)
		{
			DrawRelationEdgePassMethod2();
			DrawResult();
		}
		else	DrawScreenAlignedQuad(fbo_opacity->tex1[0],0,0,0,0,0,0);
	}
}
void RenderImplementation::DrawVertices()
{
	switch(selectedObj->GetActualSelectedObjType())
	{	
	case SelectedObject::SO_Vertex: glUniform4f(shaderobj->parameters[5]->id, sphereColor.x, sphereColor.y, sphereColor.z, sphereAlpha);	
									DrawNotSelectedVertices(selectedObj->GetVertex());
									glUniform4f(shaderobj->parameters[5]->id, selectionColor.x, selectionColor.y, selectionColor.z, selectionAlpha);	
									DrawSelectedVertex(selectedObj->GetVertex());
		break;
	default: DrawAllVertices();
		break;
	}
}
void RenderImplementation::DrawAllVertices()
{
	RenderRootedTree* rootedtree = 0;
	RenderBallonTree* ballontree = 0;
	RenderTreeMap* treemap = 0;
	RenderRadialTree* radialtree = 0;

	if(View::m_pkGraph != 0 && m_drawVertices)
	{
		glPushMatrix();
		glRotatef(rotateTree.x, 1, 0, 0);
		glRotatef(rotateTree.y, 0, 1, 0);
		glRotatef(rotateTree.z, 0, 0, 1);
		glScalef(scaleTree,scaleTree,scaleTree);

		switch(actView)
		{
		case View::VM_ROOTEDTREE: 
				if(tree[0] != 0) rootedtree = dynamic_cast<RenderRootedTree*>(tree[0]);
				if(rootedtree != 0) rootedtree->DrawVertices();
			break;
		case View::VM_BALLONTREE: 
				if(tree[1] != 0) ballontree = dynamic_cast<RenderBallonTree*>(tree[1]);
				if(ballontree != 0) ballontree->DrawVertices();
			break;
		case View::VM_TREEMAP_SLICE_AND_DICE: 
				if(tree[2] != 0) treemap = dynamic_cast<RenderTreeMap*>(tree[2]);
				if(treemap != 0) treemap->DrawVertices();
			break;
		case View::VM_TREEMAP_STRIP: 
				if(tree[3] != 0) treemap = dynamic_cast<RenderTreeMap*>(tree[3]);
				if(treemap != 0) treemap->DrawVertices();
			break;
		case View::VM_RADIALTREE: 
				if(tree[4] != 0) radialtree = dynamic_cast<RenderRadialTree*>(tree[4]);
				if(radialtree != 0) radialtree->DrawVertices();
			break;
		}
		glPopMatrix();
	}
}

void RenderImplementation::DrawSelectedVertex(VertexPtr v)
{
	RenderRootedTree* rootedtree = 0;
	RenderBallonTree* ballontree = 0;
	RenderTreeMap* treemap = 0;
	RenderRadialTree* radialtree = 0;

	if(View::m_pkGraph != 0 && m_drawVertices)
	{
		glPushMatrix();
		glRotatef(rotateTree.x, 1, 0, 0);
		glRotatef(rotateTree.y, 0, 1, 0);
		glRotatef(rotateTree.z, 0, 0, 1);
		glScalef(scaleTree,scaleTree,scaleTree);

		switch(actView)
		{
		case View::VM_ROOTEDTREE: 
				if(tree[0] != 0) rootedtree = dynamic_cast<RenderRootedTree*>(tree[0]);
				if(rootedtree != 0) rootedtree->DrawVertex(v);
			break;
		case View::VM_BALLONTREE: 
				if(tree[1] != 0) ballontree = dynamic_cast<RenderBallonTree*>(tree[1]);
				if(ballontree != 0) ballontree->DrawVertex(v);
			break;
		case View::VM_TREEMAP_SLICE_AND_DICE: 
				if(tree[2] != 0) treemap = dynamic_cast<RenderTreeMap*>(tree[2]);
				if(treemap != 0) treemap->DrawVertex(v);
			break;
		case View::VM_TREEMAP_STRIP: 
				if(tree[3] != 0) treemap = dynamic_cast<RenderTreeMap*>(tree[3]);
				if(treemap != 0) treemap->DrawVertex(v);
			break;
		case View::VM_RADIALTREE: 
				if(tree[4] != 0) radialtree = dynamic_cast<RenderRadialTree*>(tree[4]);
				if(radialtree != 0) radialtree->DrawVertex(v);
			break;
		}
		glPopMatrix();
	}
}

void RenderImplementation::DrawNotSelectedVertices(VertexPtr v)
{
	RenderRootedTree* rootedtree = 0;
	RenderBallonTree* ballontree = 0;
	RenderTreeMap* treemap = 0;
	RenderRadialTree* radialtree = 0;

	if(View::m_pkGraph != 0 && m_drawVertices)
	{
		glPushMatrix();
		glRotatef(rotateTree.x, 1, 0, 0);
		glRotatef(rotateTree.y, 0, 1, 0);
		glRotatef(rotateTree.z, 0, 0, 1);
		glScalef(scaleTree,scaleTree,scaleTree);

		switch(actView)
		{
		case View::VM_ROOTEDTREE: 
				if(tree[0] != 0) rootedtree = dynamic_cast<RenderRootedTree*>(tree[0]);
				if(rootedtree != 0) rootedtree->DrawWithoutVertex(v);
			break;
		case View::VM_BALLONTREE: 
				if(tree[1] != 0) ballontree = dynamic_cast<RenderBallonTree*>(tree[1]);
				if(ballontree != 0) ballontree->DrawWithoutVertex(v);
			break;
		case View::VM_TREEMAP_SLICE_AND_DICE: 
				if(tree[2] != 0) treemap = dynamic_cast<RenderTreeMap*>(tree[2]);
				if(treemap != 0) treemap->DrawWithoutVertex(v);
			break;
		case View::VM_TREEMAP_STRIP: 
				if(tree[3] != 0) treemap = dynamic_cast<RenderTreeMap*>(tree[3]);
				if(treemap != 0) treemap->DrawWithoutVertex(v);
			break;
		case View::VM_RADIALTREE: 
				if(tree[4] != 0) radialtree = dynamic_cast<RenderRadialTree*>(tree[4]);
				if(radialtree != 0) radialtree->DrawWithoutVertex(v);
			break;
		}
		glPopMatrix();
	}
}


void RenderImplementation::DrawGrid()
{
	RenderRootedTree* rootedtree = 0;
	RenderBallonTree* ballontree = 0;
	RenderTreeMap* treemap = 0;
	RenderRadialTree* radialtree = 0;

	if(View::m_pkGraph != 0 && m_drawGrid)
	{
		glPushMatrix();
		glRotatef(rotateTree.x, 1, 0, 0);
		glRotatef(rotateTree.y, 0, 1, 0);
		glRotatef(rotateTree.z, 0, 0, 1);
		glScalef(scaleTree,scaleTree,scaleTree);

		switch(actView)
		{
		case View::VM_ROOTEDTREE: 
				if(tree[0] != 0) rootedtree = dynamic_cast<RenderRootedTree*>(tree[0]);
				if(rootedtree != 0) rootedtree->DrawGrid();
			break;
		case View::VM_BALLONTREE: 
				if(tree[1] != 0) ballontree = dynamic_cast<RenderBallonTree*>(tree[1]);
				if(ballontree != 0) ballontree->DrawGrid();
			break;
		case View::VM_TREEMAP_SLICE_AND_DICE: 
				if(tree[2] != 0) treemap = dynamic_cast<RenderTreeMap*>(tree[2]);
				if(treemap != 0) treemap->DrawGrid();
			break;
		case View::VM_TREEMAP_STRIP: 
				if(tree[3] != 0) treemap = dynamic_cast<RenderTreeMap*>(tree[3]);
				if(treemap != 0) treemap->DrawGrid();
			break;
		case View::VM_RADIALTREE: 
				if(tree[4] != 0) radialtree = dynamic_cast<RenderRadialTree*>(tree[4]);
				if(radialtree != 0) radialtree->DrawGrid();
			break;
		}
		glPopMatrix();
	}
}
void RenderImplementation::DrawScreenAlignedQuad(int tex, int tex2, int tex3, int tex4, int tex5,int tex6,int tex7)
{
	glPushMatrix();
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
		
	glViewport(0,0,1024,768);
	glOrtho(-0.5,0.5,-0.5,0.5,1,20);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glClear (GL_DEPTH_BUFFER_BIT );

	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, tex);
	glEnable(GL_TEXTURE_2D);
		
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, tex2);
	glEnable(GL_TEXTURE_2D);
	
	glActiveTexture(GL_TEXTURE2);
	glBindTexture(GL_TEXTURE_2D, tex3);
	glEnable(GL_TEXTURE_2D);

	glActiveTexture(GL_TEXTURE3);
	glBindTexture(GL_TEXTURE_2D, tex4);
	glEnable(GL_TEXTURE_2D);

	glActiveTexture(GL_TEXTURE5);
	glBindTexture(GL_TEXTURE_2D, tex5);
	glEnable(GL_TEXTURE_2D);

	glActiveTexture(GL_TEXTURE6);
	glBindTexture(GL_TEXTURE_2D, tex6);
	glEnable(GL_TEXTURE_2D);

	glActiveTexture(GL_TEXTURE7);
	glBindTexture(GL_TEXTURE_2D, tex7);
	glEnable(GL_TEXTURE_2D);

	glTranslated(0,0,-1);
	glBegin(GL_QUADS);
		glMultiTexCoord2iARB(GL_TEXTURE0,0,0);
		glMultiTexCoord2iARB(GL_TEXTURE1,0,0);
		glMultiTexCoord2iARB(GL_TEXTURE2,0,0);
		glMultiTexCoord2iARB(GL_TEXTURE3,0,0);
		glMultiTexCoord2iARB(GL_TEXTURE3,0,0);
		glMultiTexCoord2fARB(GL_TEXTURE4,0,0);
		glMultiTexCoord2iARB(GL_TEXTURE5,0,0);
		glMultiTexCoord2iARB(GL_TEXTURE6,0,0);
		glMultiTexCoord2iARB(GL_TEXTURE7,0,0);
		glVertex3f(-0.5,-0.5,0);

		glMultiTexCoord2iARB(GL_TEXTURE0,1,0);
		glMultiTexCoord2iARB(GL_TEXTURE1,1,0);
		glMultiTexCoord2iARB(GL_TEXTURE2,1,0);
		glMultiTexCoord2iARB(GL_TEXTURE3,1,0);
		glMultiTexCoord2fARB(GL_TEXTURE4,1,0);
		glMultiTexCoord2iARB(GL_TEXTURE5,1,0);
		glMultiTexCoord2iARB(GL_TEXTURE6,1,0);
		glMultiTexCoord2iARB(GL_TEXTURE7,1,0);
		glVertex3f(0.5,-0.5,0);

		glMultiTexCoord2iARB(GL_TEXTURE0,1,1);
		glMultiTexCoord2iARB(GL_TEXTURE1,1,1);
		glMultiTexCoord2iARB(GL_TEXTURE2,1,1);
		glMultiTexCoord2iARB(GL_TEXTURE3,1,1);
		glMultiTexCoord2fARB(GL_TEXTURE4,1,1);
		glMultiTexCoord2iARB(GL_TEXTURE5,1,1);
		glMultiTexCoord2iARB(GL_TEXTURE6,1,1);
		glMultiTexCoord2iARB(GL_TEXTURE7,1,1);
		glVertex3f(0.5,0.5,0);

		glMultiTexCoord2iARB(GL_TEXTURE0,0,1);
		glMultiTexCoord2iARB(GL_TEXTURE1,0,1);
		glMultiTexCoord2iARB(GL_TEXTURE2,0,1);
		glMultiTexCoord2iARB(GL_TEXTURE3,0,1);
		glMultiTexCoord2fARB(GL_TEXTURE4,0,1);
		glMultiTexCoord2iARB(GL_TEXTURE5,0,1);
		glMultiTexCoord2iARB(GL_TEXTURE6,0,1);
		glMultiTexCoord2iARB(GL_TEXTURE7,0,1);
		glVertex3f(-0.5,0.5,0);
	glEnd();

	glPopMatrix();

	glActiveTexture(GL_TEXTURE7);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE6);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE5);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE4);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE3);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE2);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE1);
	glDisable(GL_TEXTURE_2D); 
	glActiveTexture(GL_TEXTURE0);
	glDisable(GL_TEXTURE_2D);
}



void RenderImplementation::DrawHierarchyEdges()
{
	DrawHierarchyEdges(true);
}
void RenderImplementation::DrawHierarchyEdges(bool noshadow)
{
	if(m_view != 0 && m_drawHierarchicEdges)
	{
		//noshadow = false;
		glPushMatrix();
		glRotatef(rotateTree.x, 1, 0, 0);
		glRotatef(rotateTree.y, 0, 1, 0);
		glRotatef(rotateTree.z, 0, 0, 1); 
		glScalef(scaleTree,scaleTree,scaleTree);

		//Draw Hierarchy edges
		glEnableClientState(GL_VERTEX_ARRAY);
		glEnableClientState(GL_COLOR_ARRAY);
		if(noshadow) glEnableClientState(GL_TEXTURE_COORD_ARRAY);

		glBindBuffer( GL_ARRAY_BUFFER, m_view[actView]->vbohierarchylines_vert );
		glVertexPointer( 3, GL_FLOAT, 0, 0 );		// Set The Vertex Pointer To The Vertex Buffer
		glBindBuffer(GL_ARRAY_BUFFER, m_view[actView]->vbohierarchylines_color);
		glColorPointer(4,GL_FLOAT, 0, 0);
		
		if(noshadow)
		{
			glActiveTexture(GL_TEXTURE7);
			glBindBuffer(GL_ARRAY_BUFFER, m_view[actView]->vboindex_picking_ind);
			glTexCoordPointer(4, GL_FLOAT, 0, 0);
		}
		glPushMatrix();
		glDrawArrays( GL_LINES, 0, numberofhierarchyedges*2 );
		glPopMatrix();

		glDisableClientState(GL_VERTEX_ARRAY);
		glDisableClientState(GL_COLOR_ARRAY);
		if(noshadow) glDisableClientState(GL_TEXTURE_COORD_ARRAY);
		glPopMatrix();
	}
}

void RenderImplementation::DrawRelationEdges()
{
	/* wei nicht wo man das am besten zusammenbaut... 
		Die Bsplines sind alle in einem Vector zugriff erfolgt ber
		QVector<BSpline*>* vector = splineSet->getSplineVector(actuelleview)
		Die Splines die im Vector sind, sind schon der lnge nach absteigend geordnet.
		vector->at(i)->getXValues() // getYValues() // getZValues() fallst sie anders brauchst...
		siehe unten rufe die draw methode auf, momentan ainfach nur gezeichnet, glaub aber das da was noch nicht ganz stimmt, 
		bezglich der blending modes oder so, sieht so aus als wrde er gewisse einstellungen nicht bernehmen....

		ach ja das mit dem Beta, einfach splineSet->setNewBundlingValue(float beta), dann werden fr alle Darstellungen die Splines neu berechnet...
	*/
	
	if(m_drawRelationEdges && m_view != 0)
	{
		glPushMatrix();
		glRotatef(rotateTree.x, 1, 0, 0);
		glRotatef(rotateTree.y, 0, 1, 0);
		glRotatef(rotateTree.z, 0, 0, 1);
		glScalef(scaleTree,scaleTree,scaleTree);
		QVector<BSpline*>* vector = splineSet->getSplineVector(actView);
		for (int i = 0; i < vector->size(); i++)
		{
			vector->at(i)->draw();
		}
		glPopMatrix();
	}
}

void RenderImplementation::BeginCamera()
{
	BeginCamera(camera);
}

void RenderImplementation::BeginCamera(Camera* cam)
{
	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf(&cam->getProjectionMatrix()[0][0]);

	glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(&cam->getViewingMatrix()[0][0]);

	glPushMatrix();
	//glTranslatef(0.0, 0.0, 1.0);
}

void RenderImplementation::EndCamera()
{
	glPopMatrix();	
}

void RenderImplementation::BeginTextures(int tex, int tex2, int tex3, int tex4, int tex5,int tex6,int tex7)
{
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, tex);
	glEnable(GL_TEXTURE_2D);
		
	glActiveTexture(GL_TEXTURE1);
	glBindTexture(GL_TEXTURE_2D, tex2);
	glEnable(GL_TEXTURE_2D);
	
	glActiveTexture(GL_TEXTURE2);
	glBindTexture(GL_TEXTURE_2D, tex3);
	glEnable(GL_TEXTURE_2D);

	glActiveTexture(GL_TEXTURE3);
	glBindTexture(GL_TEXTURE_2D, tex4);
	glEnable(GL_TEXTURE_2D);

	glActiveTexture(GL_TEXTURE5);
	glBindTexture(GL_TEXTURE_2D, tex5);
	glEnable(GL_TEXTURE_2D);

	glActiveTexture(GL_TEXTURE6);
	glBindTexture(GL_TEXTURE_2D, tex6);
	glEnable(GL_TEXTURE_2D);

	glActiveTexture(GL_TEXTURE7);
	glBindTexture(GL_TEXTURE_2D, tex7);
	glEnable(GL_TEXTURE_2D);
}

void RenderImplementation::EndTextures()
{
	glActiveTexture(GL_TEXTURE7);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE6);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE5);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE4);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE3);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE2);
	glDisable(GL_TEXTURE_2D);
	glActiveTexture(GL_TEXTURE1);
	glDisable(GL_TEXTURE_2D); 
	glActiveTexture(GL_TEXTURE0);
	glDisable(GL_TEXTURE_2D);
}

void RenderImplementation::LoadFont()								// Build Our Font Display List
{
	float	cx=0;											// Holds Our X Character Coord
	float	cy=0;											// Holds Our Y Character Coord

	Image* img = new Image();
	img->LoadImg(IL_PNG, "imgs/Font.png");
	glGenTextures(1, &tex_font);
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, tex_font);
	glEnable(GL_TEXTURE_2D);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, img->GetWidth(), img->GetHeight(), 0, GL_RGBA, GL_UNSIGNED_BYTE, img->GetData());
	glFinish();
	glFlush();
	glDisable(GL_TEXTURE_2D);
	delete(img);

	dl_font=glGenLists(256);								// Creating 256 Display Lists
	for (int loop=0; loop<256; loop++)					// Loop Through All 256 Lists
	{
		cx=float(loop%16)/16.0f;						// X Position Of Current Character
		cy=float(loop/16)/16.0f;						// Y Position Of Current Character
		glColor4f(1.0f,1.0f,1.0f,0.0f);
		glNewList(dl_font+loop,GL_COMPILE);				// Start Building A List
			glBegin(GL_QUADS);							// Use A Quad For Each Character
				glMultiTexCoord2f(GL_TEXTURE0, cx,1-cy-0.0625f);			// Texture Coord (Bottom Left)
				glVertex2i(0,0);						// Vertex Coord (Bottom Left)
				glMultiTexCoord2f(GL_TEXTURE0, cx+0.0625f,1-cy-0.0625f);	// Texture Coord (Bottom Right)
				glVertex2i(16,0);						// Vertex Coord (Bottom Right)
				glMultiTexCoord2f(GL_TEXTURE0, cx+0.0625f,1-cy);			// Texture Coord (Top Right)
				glVertex2i(16,16);						// Vertex Coord (Top Right)
				glMultiTexCoord2f(GL_TEXTURE0, cx,1-cy);					// Texture Coord (Top Left)
				glVertex2i(0,16);						// Vertex Coord (Top Left)
			glEnd();									// Done Building Our Quad (Character)
			glTranslated(7.4,0,0);						// Move To The Right Of The Character
		glEndList();									// Done Building The Display List
	}													// Loop Until All 256 Are Built
}

void RenderImplementation::UnloadFont()									// Delete The Font From Memory
{
	glDeleteLists(dl_font,256);							// Delete All 256 Display Lists
}

void RenderImplementation::DrawFont(GLint x, GLint y, std::string string, int set,float scale)	// Where The Printing Happens
{
	if (set>1)	set=1;

	glDisable(GL_DEPTH_TEST);							// Disables Depth Testing
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPushMatrix();										// Store The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix
	glOrtho(0,1024,0,768,-1,1);							// Set Up An Ortho Screen
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPushMatrix();										// Store The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix
	glTranslated(x,y,0);								// Position The Text (0,0 - Bottom Left)
		glListBase(dl_font-32+(128*set));					// Choose The Font Set (0 or 1)
		glScalef(scale,0.5f+scale,0.5f+scale);
		glCallLists(string.length(),GL_BYTE,string.c_str());		// Write The Text To The Screen
	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glPopMatrix();										// Restore The Old Projection Matrix
	glEnable(GL_DEPTH_TEST);							// Enables Depth Testing
}
void RenderImplementation::DrawHUD(int time)
{
	glColor4f(1,1,1,1);
	if(time < 1) time = 1;
	float ffps = 1000.0f/(float)(time);
	//std::cout << "TIME: " << time << "FFPS: " << ffps << std::endl;
	int fps = ffps;

	/************************************************************************************/
	glActiveTexture(GL_TEXTURE0);
	glBindTexture(GL_TEXTURE_2D, tex_font);			// Select Our Font Texture
	glEnable(GL_TEXTURE_2D);							// Enable 2D Texture Mapping

	glDisable(GL_DEPTH_TEST);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);					
	glEnable(GL_BLEND);									// Enable Blending
	glLoadIdentity();									// Reset The View

	char buffer[6];	for(int i = 0; i < 6; i++) buffer[i] = ' ';
	char buffer_id[6];	for(int i = 0; i < 6; i++) buffer_id[i] = ' ';
	_snprintf_s(buffer, 6, "%d", fps);

	glColor4f(1.0f,1.0f,1.0f,1.0f);
	switch(selectedObj->GetActualSelectedObjType())
	{
	case SelectedObject::SO_Vertex:
	{
		VertexPtr obj = selectedObj->GetVertex();
		_snprintf_s(buffer_id, 6, "%d", obj->GetID());

		DrawFont(5,730,"Type: Vertex",1,0.75f);
		DrawFont(5,710,"ID: "+ std::string(buffer_id),1,0.75f);
		DrawFont(5,690,"Name: " + obj->GetName(),1,0.75f);
		break;
	}
	case SelectedObject::SO_HEDGE:
	{		
		HierarchyPtr obj = selectedObj->GetHierarchyEdge();
		_snprintf_s(buffer_id, 6, "%d", obj->GetID());

		char buffer_startid[6];	for(int i = 0; i < 6; i++) buffer_startid[i] = ' '; _snprintf_s(buffer_startid, 6, "%d", obj->GetStartEdge()->GetID());
		char buffer_endid[6];	for(int i = 0; i < 6; i++) buffer_endid[i] = ' '; _snprintf_s(buffer_endid, 6, "%d", obj->GetEndEdge()->GetID());

		DrawFont(5,730,"Type: Hierarchy Edge",1,0.75f);
		DrawFont(5,710,"ID: "+ std::string(buffer_id),1,0.75f);
		DrawFont(5,690,"Start: " + std::string(buffer_startid) + " " + obj->GetStartEdge()->GetName(),1,0.75f);
		DrawFont(5,670,"End: " + std::string(buffer_endid) + " " + obj->GetEndEdge()->GetName(),1,0.75f);
		break;
	}
	case SelectedObject::SO_REDGE:
	{
		RelationPtr obj = selectedObj->GetRelationEdge();
		_snprintf_s(buffer_id, 6, "%d", obj->GetID());

		char buffer_startid[6];	for(int i = 0; i < 6; i++) buffer_startid[i] = ' '; _snprintf_s(buffer_startid, 6, "%d", obj->GetStartEdge()->GetID());
		char buffer_endid[6];	for(int i = 0; i < 6; i++) buffer_endid[i] = ' '; _snprintf_s(buffer_endid, 6, "%d", obj->GetEndEdge()->GetID());

		DrawFont(5,730,"Type: Relation Edge",1,0.75f);
		DrawFont(5,710,"ID: "+ std::string(buffer_id),1,0.75f);
		DrawFont(5,690,"Start: " + std::string(buffer_startid) + " " + obj->GetStartEdge()->GetName(),1,0.75f);
		DrawFont(5,670,"End: " + std::string(buffer_endid) + " " + obj->GetEndEdge()->GetName(),1,0.75f);
		break;
	}
	case SelectedObject::SO_NONE:
	{
		DrawFont(5,730,"Type: ",1,0.75f);
		DrawFont(5,710,"ID:",1,0.75f);
		break;
	}
	}

	glColor4f(1.0f,1.0f,1.0f,1.0f);
	DrawFont(960,713,buffer,0,1.8f);

	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);
	/************************************************************************************/
}


//---------------
// Picking Stuff
//---------------
void RenderImplementation::Picking(int curserX, int curserY)
{
	if(View::m_pkGraph != 0)
	{
		glFinish();
		std::cout << "Mouse X: " << curserX << " y: " << curserY << std::endl;
		std::cout << "MAX X: " << camera->getScreenWidth() << " y: " << camera->getScreenHeight() << std::endl;
		curserX -= 1;				curserX = max(0, curserX); curserX = min(1023, curserX);
		curserY -= 1;				curserY = max(0, curserY); curserY = min(767, curserY);		curserY = 767-(curserY);	

		//So ist es viel schneller, als wenn ich die ganze Textur lese.
		//Eigentlich eh logisch...
		float erg[4];
		fbo_opacity->Bind();
		glReadBuffer(GL_COLOR_ATTACHMENT1_EXT);
		if(curserX >= 0 && curserY >= 0 && curserX < 1024 && curserY < 768)
		{
			glReadPixels(curserX,curserY,1,1,GL_RGBA , GL_FLOAT, erg); 
			if(erg[1] == 0.0f && erg[2] == 0.0f && erg[0] < 1.0f)		
			{
				float selectdid = erg[0] * (float)(View::m_pkGraph->GetNumberOfVertices());
				selectedObj->SetSelectedObj(View::m_pkGraph->GetVertexWithID(selectdid));
				UpdateSelection();
			}
			if(erg[0] == 0.0f && erg[2] == 0.0f && erg[1] < 1.0f)		
			{
				float selectdid = erg[1] * (float)(View::m_pkGraph->GetNumberOfHiearchicEdges());
				selectedObj->SetSelectedObj(View::m_pkGraph->GetHierarchyEdgeWithID(selectdid));
				UpdateSelection();
			}
			if(erg[0] == 0.0f && erg[1] == 0.0f  && erg[2] < 1.0f)		
			{
				float helper = erg[2] * (float)(View::m_pkGraph->GetNumberOfRelationEdges());
				QVector<BSpline*>* vector = splineSet->getSplineVector(actView);
				int selectdid = vector->at((int)(helper))->GetID();				
				selectedObj->SetSelectedObj(View::m_pkGraph->GetRelationEdgeWithID(selectdid), helper);
				UpdateSelection();
		
			}
			std::cout << "ERG: " << erg[0] << " " << erg[1] << " " << erg[2] << " " << erg[3] << std::endl;
		}
		glReadBuffer(GL_FRONT);
		fbo_opacity->Unbind();
	}
}

void RenderImplementation::UpdateSelection()
{
	std::cout << "Update Selection" << std::endl;
	switch(selectedObj->GetActualSelectedObjType())
	{	
	case SelectedObject::SO_Vertex: if(selectedObj->m_oldhelper >= 0) UpdateRelationEdgeColor(relationStartColor, relationStartAlpha, relationEndColor, relationEndAlpha, selectedObj->m_oldhelper);
									if(selectedObj->GetOldHierarchyEdge() != 0) UpdateHierarchyEdgeColorSelectionAt(hierarchyColor, hierarchyAlpha, hierarchyColor, hierarchyAlpha, selectedObj->GetOldHierarchyEdge()->GetID());
		break;
	case SelectedObject::SO_HEDGE:	if(selectedObj->m_oldhelper >= 0) UpdateRelationEdgeColor(relationStartColor, relationStartAlpha, relationEndColor, relationEndAlpha, selectedObj->m_oldhelper);
									if(selectedObj->GetOldHierarchyEdge() != 0) UpdateHierarchyEdgeColorSelectionAt(hierarchyColor, hierarchyAlpha, hierarchyColor, hierarchyAlpha, selectedObj->GetOldHierarchyEdge()->GetID());
									UpdateHierarchyEdgeColorSelectionAt(hierarchyColor, hierarchyAlpha, selectionColor, selectionAlpha, selectedObj->GetHierarchyEdge()->GetID());
		break;
	case SelectedObject::SO_REDGE:	if(selectedObj->GetOldHierarchyEdge() != 0) UpdateHierarchyEdgeColorSelectionAt(hierarchyColor, hierarchyAlpha, hierarchyColor, hierarchyAlpha, selectedObj->GetOldHierarchyEdge()->GetID());
									if(selectedObj->m_oldhelper >= 0) UpdateRelationEdgeColor(relationStartColor, relationStartAlpha, relationEndColor, relationEndAlpha, selectedObj->m_oldhelper);
									UpdateRelationEdgeColor(selectionColor, selectionAlpha, selectionColor, selectionAlpha, selectedObj->m_helper);
		break;
	}
}

void RenderImplementation::UpdateSphereColor(vec3 color, float alpha)
{
	sphereColor = color;
	sphereAlpha = alpha;

}

void RenderImplementation::UpdateHierarchyEdgeColor(vec3 color, float alpha)
{
	for(unsigned int actView = 0;m_view != 0 && actView < View::VM_QUANTITY; actView++)
	{
		// bind then map the VBO
		glBindBufferARB(GL_ARRAY_BUFFER_ARB, m_view[actView]->vbohierarchylines_color);
		float* ptr = (float*)glMapBufferARB(GL_ARRAY_BUFFER_ARB, GL_WRITE_ONLY_ARB);

		// if the pointer is valid(mapped), update VBO
		if(ptr)
		{	
			for(unsigned int i = 0; i < View::m_pkGraph->GetNumberOfHiearchicEdges()*2; i++)
			{
				ptr[(4*i)+0] = color.x;
				ptr[(4*i)+1] = color.y;
				ptr[(4*i)+2] = color.z;
				ptr[(4*i)+3] = alpha;
			}
			glUnmapBufferARB(GL_ARRAY_BUFFER_ARB); // unmap it after use
		}
	}
}


void RenderImplementation::UpdateHierarchyEdgeColorAt(vec3 color, float alpha, unsigned int i)
{
	std::cout << "UpdateHierarchyEdgeColorAt" << std::endl;
	// bind then map the VBO
	glBindBufferARB(GL_ARRAY_BUFFER_ARB, m_view[actView]->vbohierarchylines_color);
	float* ptr = (float*)glMapBufferARB(GL_ARRAY_BUFFER_ARB, GL_WRITE_ONLY_ARB);

	i = i*2;

	// if the pointer is valid(mapped), update VBO
	if(ptr)
	{	
		ptr[(4*i)+0] = color.x;
		ptr[(4*i)+1] = color.y;
		ptr[(4*i)+2] = color.z;
		ptr[(4*i)+3] = alpha;
		ptr[(4*i)+4] = color.x;
		ptr[(4*i)+5] = color.y;
		ptr[(4*i)+6] = color.z;
		ptr[(4*i)+7] = alpha;
		
		glUnmapBufferARB(GL_ARRAY_BUFFER_ARB); // unmap it after use
	}
}

void RenderImplementation::UpdateHierarchyEdgeColorSelectionAt(vec3 color, float alpha, vec3 sele_color, float sele_alpha, unsigned int at)
{
	for(unsigned int actView = 0;m_view != 0 && actView < View::VM_QUANTITY; actView++)
	{
		// bind then map the VBO
		glBindBufferARB(GL_ARRAY_BUFFER_ARB, m_view[actView]->vbohierarchylines_color);
		float* ptr = (float*)glMapBufferARB(GL_ARRAY_BUFFER_ARB, GL_WRITE_ONLY_ARB);

		// if the pointer is valid(mapped), update VBO
		if(ptr)
		{	
			for(unsigned int i = 0; i < View::m_pkGraph->GetNumberOfHiearchicEdges(); i+=1)
			{
				if(at == i)
				{
					ptr[(8*i)+0] = sele_color.x;
					ptr[(8*i)+1] = sele_color.y;
					ptr[(8*i)+2] = sele_color.z;
					ptr[(8*i)+3] = sele_alpha;
					ptr[(8*i)+4] = sele_color.x;
					ptr[(8*i)+5] = sele_color.y;
					ptr[(8*i)+6] = sele_color.z;
					ptr[(8*i)+7] = sele_alpha;
				}
				else
				{
					ptr[(8*i)+0] = color.x;
					ptr[(8*i)+1] = color.y;
					ptr[(8*i)+2] = color.z;
					ptr[(8*i)+3] = alpha;
					ptr[(8*i)+4] = color.x;
					ptr[(8*i)+5] = color.y;
					ptr[(8*i)+6] = color.z;
					ptr[(8*i)+7] = alpha;
				}
			}
			glUnmapBufferARB(GL_ARRAY_BUFFER_ARB); // unmap it after use
		}
	}
}

void RenderImplementation::UpdateRelationEdgeColor(vec3 color, float alpha, vec3 sele_color, float sele_alpha, unsigned int at)
{	
	QVector<BSpline*>* vector = splineSet->getSplineVector(actView);
	vector->at(at)->UpdateColor(vec4(color.x,color.y, color.z, alpha), vec4(sele_color.x,sele_color.y, sele_color.z, sele_alpha));
	
}
void RenderImplementation::UpdateRelationEdgeColor(vec3 startcolor, float alphastart, vec3 endcolor, float alphaend)
{
	if(m_drawRelationEdges && m_view != 0)
	{
		glPushMatrix();
		glRotatef(rotateTree.x, 1, 0, 0);
		glRotatef(rotateTree.y, 0, 1, 0);
		glRotatef(rotateTree.z, 0, 0, 1);
		glScalef(scaleTree,scaleTree,scaleTree);
		QVector<BSpline*>* vector = splineSet->getSplineVector(actView);
		for (int i = 0; i < vector->size(); i++)
		{
			vector->at(i)->UpdateColor(vec4(startcolor.x,startcolor.y,startcolor.z, alphastart), vec4(endcolor.x,endcolor.y,endcolor.z, alphaend));
		}
		glPopMatrix();
	}
}

void RenderImplementation::UpdateSelectionColor(vec3 color, float alpha)
{
	UpdateSelection();
}	

Matrix RenderImplementation::GetModelViewMatrix()
{
	float modelview[16];
	glGetFloatv(GL_MODELVIEW_MATRIX , modelview);
	return Matrix(modelview);	
}

Matrix RenderImplementation::GetProjectionMatrix()
{
	float projection[16];
	glGetFloatv(GL_PROJECTION_MATRIX , projection);
	return Matrix(projection);	
}