#include "Renderer.h"

Renderer::Renderer(void)
{
}

Renderer::~Renderer(void)
{
}

void Renderer::SetBlending(bool value)
{
	m_blending = value;
}

void Renderer::UpdateStates()
{
	actView = RenderStates::actView;
	this->blendingmode = RenderStates::blendingMode;
	SetBlending(RenderStates::blendingon);

	rotateTree = vec3(RenderStates::RotateX, RenderStates::RotateY, RenderStates::RotateZ);
	scaleTree = RenderStates::Scale;
	scaleTree /= 10;
	if(scaleTree <= 5)
	{
		scaleTree = 6.f-scaleTree;
		scaleTree = 1.f/scaleTree;
	}
	else	scaleTree = scaleTree-4.f;

	if(RenderStates::lighton)	m_light->On();
	else						m_light->Off();
	if(RenderStates::enable_shadow) m_light->ShadowOn();
	else							m_light->ShadowOff();
	m_light->SetType(RenderStates::lighttype);
	//m_light->SetPosition(RenderStates::lightpos);
	m_light->SetSpotDirection(RenderStates::spotdir);
	m_light->SetShadowElements(RenderStates::shadow);
	m_light->SetRadius((float)(RenderStates::LightRadius));
	m_light->SetAttentuate((float)(RenderStates::LightAttenuate)/10.f);
	SetEnabledElements(RenderStates::drawNodes);

	dynamic_cast<RenderTreeMap*>(tree[2])->m_cushiontexture = RenderStates::cushionTexture;
	dynamic_cast<RenderTreeMap*>(tree[3])->m_cushiontexture = RenderStates::cushionTexture;

	vec3 old_sphereColor = sphereColor;
	float old_sphereAlpha = sphereAlpha;
	vec3 old_hierarchyColor = hierarchyColor;
	float old_hierarchyAlpha = hierarchyAlpha;
	vec3 old_relationStartColor = relationStartColor;
	float old_relationStartAlpha = relationStartAlpha;
	vec3 old_relationEndColor = relationEndColor;
	float old_relationEndAlpha = relationEndAlpha;
	vec3 old_selectionColor = selectionColor;
	float old_selectionAlpha = selectionAlpha;

	sphereColor = RenderStates::vertexColor;
	sphereAlpha = RenderStates::vertexAlpha;
	hierarchyColor = RenderStates::hierarchicColor;
	hierarchyAlpha = RenderStates::herarchicAlpha;
	relationStartColor = RenderStates::startRelationColor;
	relationStartAlpha = RenderStates::startRelationAlpha;
	relationEndColor = RenderStates::endRelationColor;
	relationEndAlpha = RenderStates::endRelationAlpha;
	selectionColor = RenderStates::selectionColor;
	selectionAlpha = RenderStates::selectionAlpha;

	if(sphereColor.x != old_sphereColor.x				|| sphereColor.y != old_sphereColor.y				|| sphereColor.z != old_sphereColor.z				|| old_sphereAlpha != sphereAlpha)				UpdateSphereColor(sphereColor, sphereAlpha);
	if(hierarchyColor.x != old_hierarchyColor.x			|| hierarchyColor.y != old_hierarchyColor.y			|| hierarchyColor.z != old_hierarchyColor.z			|| old_hierarchyAlpha != hierarchyAlpha)		UpdateHierarchyEdgeColor(hierarchyColor, hierarchyAlpha);
	if(relationStartColor.x != old_relationStartColor.x || relationStartColor.y != old_relationStartColor.y || relationStartColor.z != old_relationStartColor.z )UpdateRelationEdgeColor(relationStartColor, relationStartAlpha, relationEndColor, relationEndAlpha);
	if(relationEndColor.x != old_relationEndColor.x		|| relationEndColor.y != old_relationEndColor.y		|| relationEndColor.z != old_relationEndColor.z		)	UpdateRelationEdgeColor(relationStartColor, relationStartAlpha, relationEndColor, relationEndAlpha);
	if(selectionColor.x != old_selectionColor.x			|| selectionColor.y != old_selectionColor.y			|| selectionColor.z != old_selectionColor.z			|| old_selectionAlpha != selectionAlpha)		UpdateSelectionColor(selectionColor, selectionAlpha);

	if(old_relationStartAlpha != relationStartAlpha || old_relationEndAlpha != relationEndAlpha)
	{

	}

	if(RenderStates::beta != m_beta) 
	{
		m_beta = RenderStates::beta;
		splineSet->setNewBundlingValue(m_beta);
	}

	if(RenderStates::mirrorRelationEdges != m_mirrorrelationedges)
	{

		m_mirrorrelationedges = RenderStates::mirrorRelationEdges;
	}
}

void Renderer::SetEnabledElements(int elements)
{
	switch(elements)
	{
	case 0:	m_drawVertices = false;
			m_drawHierarchicEdges = false;
			m_drawRelationEdges = false;
			m_drawGrid = false;
		break;
	case 1:	m_drawVertices = true;
			m_drawHierarchicEdges = false;
			m_drawRelationEdges = false;
			m_drawGrid = false;
		break;
	case 2:	m_drawVertices = false;
			m_drawHierarchicEdges = true;
			m_drawRelationEdges = false;
			m_drawGrid = false;
		break;
	case 3:	m_drawVertices = true;
			m_drawHierarchicEdges = true;
			m_drawRelationEdges = false;
			m_drawGrid = false;
		break;
	case 4:	m_drawVertices = false;
			m_drawHierarchicEdges = false;
			m_drawRelationEdges = true;
			m_drawGrid = false;
		break;
	case 5:	m_drawVertices = true;
			m_drawHierarchicEdges = false;
			m_drawRelationEdges = true;
			m_drawGrid = false;
		break;
	case 6:	m_drawVertices = false;
			m_drawHierarchicEdges = true;
			m_drawRelationEdges = true;
			m_drawGrid = false;
		break;
	case 7:	m_drawVertices = true;
			m_drawHierarchicEdges = true;
			m_drawRelationEdges = true;
			m_drawGrid = false;
		break;
	case 8:	m_drawVertices = false;
			m_drawHierarchicEdges = false;
			m_drawRelationEdges = false;
			m_drawGrid = true;
		break;
	case 9:	m_drawVertices = true;
			m_drawHierarchicEdges = false;
			m_drawRelationEdges = false;
			m_drawGrid = true;
		break;
	case 10:m_drawVertices = false;
			m_drawHierarchicEdges = true;
			m_drawRelationEdges = false;
			m_drawGrid = true;
		break;
	case 11:m_drawVertices = true;
			m_drawHierarchicEdges = true;
			m_drawRelationEdges = false;
			m_drawGrid = true;
		break;
	case 12:m_drawVertices = false;
			m_drawHierarchicEdges = false;
			m_drawRelationEdges = true;
			m_drawGrid = true;
		break;
	case 13:m_drawVertices = true;
			m_drawHierarchicEdges = false;
			m_drawRelationEdges = true;
			m_drawGrid = true;
		break;
	case 14:m_drawVertices = false;
			m_drawHierarchicEdges = true;
			m_drawRelationEdges = true;
			m_drawGrid = true;
		break;
	case 15:m_drawVertices = true;
			m_drawHierarchicEdges = true;
			m_drawRelationEdges = true;
			m_drawGrid = true;
		break;
	}
}