#include "RootedTree.h"
#include <glm/gtc/matrix_transform.hpp> //translate, rotate, scale, perspective

/*
Rooted Tree:


				  *
			 /		\
			*		 *
	      / |    \	  \
	     *  *     *	   *
		   /|\   /|\
		  * * * * * *

  Number of Levels: 4
  Number of Vertices without childs: 8
*/

RootedTree::RootedTree()
{
}

RootedTree::~RootedTree(void)
{
}

void RootedTree::Calculate()
{
	std::vector<VertexPtr> verticesWithoutChilds  = m_pkGraph->GetAllVerticesWithoutChild();
	//Nun wei ich wieviele Vertices ohne Child gibt, und wieviele Ebenen
	int numberOfVerticesWithoutChilds = verticesWithoutChilds.size();
	int numberOfLevels = m_pkGraph->GetNumberOfHierarchies();

	std::vector<unsigned int> parents;
	std::vector<unsigned int> parents_cpy;
	parents.resize(0);
	//Setzt alle unteren mal korrekt.
	for(double i = 0; i < verticesWithoutChilds.size(); i++)
	{
		SetGUIVertexForView(verticesWithoutChilds[(unsigned int)i]->GetID(), GUIVertex(vec3(i/(double)(numberOfVerticesWithoutChilds), (double)(verticesWithoutChilds[i]->GetHierarchyLevel())/(double)numberOfLevels, 0.0),vec3(1,0,0),0.2));
		parents.push_back(verticesWithoutChilds[i]->GetParent()->GetID());
	}

	vec3 startpos (0.0,1.0,0.0);

	while(parents.size() > 0)
	{
		parents_cpy.clear();

		for(unsigned int i = 0; i < parents.size(); i++)
		{
			VertexPtr actVertex = m_pkGraph->GetVertexWithID(parents[i]);
			glm::vec3 pos;

			for(unsigned int j = 0; j < actVertex->GetNumberChildVertices(); j++)
			{
				pos += GetGUIVertexForView(actVertex->GetChildVertexAt(j)->GetID()).GetPosition();
			}
			pos /= (double)(actVertex->GetNumberChildVertices());
			SetGUIVertexForView(actVertex->GetID(), GUIVertex(vec3(pos.x,(double)(actVertex->GetHierarchyLevel())/(double)(numberOfLevels),0.0),vec3(1,0,0),0.2 ));

			if(actVertex->GetParent() != 0)		parents_cpy.push_back(actVertex->GetParent()->GetID());
		}

		parents = parents_cpy;
	}
}
