#include "SplineSet.h"
#include "View.h"
#include <iostream>
#include <QtAlgorithms>


SplineSet::SplineSet(int numberOfHierarchies)
{
	pointsPerSpline = 50;
	bundlingBeta = 0.0f;
	this->numberOfHierarchies = numberOfHierarchies;
	basisFunktionArray = new BasisFunktion* [2 * numberOfHierarchies - 1];

	for (int i = 0; i < 2*numberOfHierarchies - 1; i++)
	{
		basisFunktionArray[i] = NULL;
	}
	
	rootedBSplineVector = new QVector<BSpline*>();
	treeMapSDBSplineVector = new QVector<BSpline*>();
	ballonBSplineVector = new QVector<BSpline*>();
	TreeMapStripSplineVector = new QVector<BSpline*>();
	RadialTreeSplineVector = new QVector<BSpline*>();

}

SplineSet::~SplineSet()
{
	rootedBSplineVector->clear();
	treeMapSDBSplineVector->clear();
	ballonBSplineVector->clear();
	TreeMapStripSplineVector->clear();
	RadialTreeSplineVector->clear();

	if(rootedBSplineVector) delete rootedBSplineVector;
	if(treeMapSDBSplineVector) delete treeMapSDBSplineVector;
	if(ballonBSplineVector) delete ballonBSplineVector;
	if(TreeMapStripSplineVector) delete TreeMapStripSplineVector;
	if(RadialTreeSplineVector) delete RadialTreeSplineVector;

	for (int i = 0; i < 2 * numberOfHierarchies - 1; i++)
	{
		if (basisFunktionArray[i]) delete basisFunktionArray[i];
	}
	if (basisFunktionArray) delete [] basisFunktionArray;
}

void SplineSet::setNewBundlingValue(float bundlingBeta)
{
	this->bundlingBeta = bundlingBeta;
	
	QVector<BSpline*>::iterator it;

	for ( it = rootedBSplineVector->begin(); it != rootedBSplineVector->end(); it++)
	{
		(*it)->recalculate(bundlingBeta);
	}

	for ( it = treeMapSDBSplineVector->begin(); it != treeMapSDBSplineVector->end(); it++)
	{
		(*it)->recalculate(bundlingBeta);
	}

	for ( it = ballonBSplineVector->begin(); it != ballonBSplineVector->end(); it++)
	{
		(*it)->recalculate(bundlingBeta);
	}
	for ( it = TreeMapStripSplineVector->begin(); it != TreeMapStripSplineVector->end(); it++)
	{
		(*it)->recalculate(bundlingBeta);
	}
	for ( it = RadialTreeSplineVector->begin(); it != RadialTreeSplineVector->end(); it++)
	{
		(*it)->recalculate(bundlingBeta);
	}
}

void SplineSet::addBSpline(QVector<QVector3D>* polygon,int size, int view, float id)
{
	if(basisFunktionArray[size-1] == NULL)
	{
		basisFunktionArray[size-1] = new BasisFunktion(size, pointsPerSpline);
		//cout << "addSPline: if basis Funktion " << size-1 << endl;
	}

	// fill two Arrays with the Values of polygon
	/*
	//Das produziert einen Fehler.
	//Die Arrays werden mit Anzahl der Elemente initialisiert und nicht mit dem grtmglichen Wert.
	//Im Debug und Release Modus macht es nichts, aber sobald man auf BSModus ist, fhrt es zu ner Exception.
	float* controlPolygonX = new float[size-1];
	float* controlPolygonY = new float[size-1];
	float* controlPolygonZ = new float[size-1];
	*/
	float* controlPolygonX = new float[size];
	float* controlPolygonY = new float[size];
	float* controlPolygonZ = new float[size];

	for (int i = 0; i<size; i++)
	{
		controlPolygonX[i] = polygon->at(i).x();
		controlPolygonY[i] = polygon->at(i).y();
		controlPolygonZ[i] = polygon->at(i).z();
		// cout << "Controlpoygon x: " << controlPolygonX[i] << " y: " << controlPolygonY[i] << " z: "<< controlPolygonZ[i] << endl;
	}

	//cout <<"addSpline: actual view " << view << " and controlpolygon size " << polygon->size() << " pointsperspline " << pointsPerSpline << "size" << size <<endl;   
	//cout << "basisFunktion " << basisFunktionArray[size - 1] << endl;
	if(view == View::VM_BALLONTREE)
	{
		ballonBSplineVector->push_back(new BSpline(basisFunktionArray[size - 1],
			controlPolygonX, controlPolygonY, controlPolygonZ, size, id));
//			controlPolygonX, controlPolygonY, controlPolygonZ, size));
	}else if( view == View::VM_ROOTEDTREE)
	{
		rootedBSplineVector->push_back(new BSpline(basisFunktionArray[size - 1],
			controlPolygonX, controlPolygonY, controlPolygonZ, size, id));
//			controlPolygonX, controlPolygonY, controlPolygonZ, size));
	}else if(view == View::VM_TREEMAP_SLICE_AND_DICE)
	{
		treeMapSDBSplineVector->push_back(new BSpline(basisFunktionArray[size - 1],
			controlPolygonX, controlPolygonY, controlPolygonZ, size, id));
//			controlPolygonX, controlPolygonY, controlPolygonZ, size));
	}else if(view == View::VM_TREEMAP_STRIP)
	{
		TreeMapStripSplineVector->push_back(new BSpline(basisFunktionArray[size - 1],
			controlPolygonX, controlPolygonY, controlPolygonZ, size, id));
//			controlPolygonX, controlPolygonY, controlPolygonZ, size));
	
	}else if(view == View::VM_RADIALTREE)
	{
		RadialTreeSplineVector->push_back(new BSpline(basisFunktionArray[size - 1],
			controlPolygonX, controlPolygonY, controlPolygonZ, size, id));
//			controlPolygonX, controlPolygonY, controlPolygonZ, size));
	}
	else{
		std::cout << "Fehler bei AddSpline kein zulssiger View" << std::endl;
	}
}

QVector<BSpline*>* SplineSet::getSplineVector(int view)
{
	if(view == View::VM_BALLONTREE)
	{
		return ballonBSplineVector;

	}else if( view == View::VM_ROOTEDTREE)
	{
		return rootedBSplineVector;

	}else if(view == View::VM_TREEMAP_SLICE_AND_DICE)
	{
		return treeMapSDBSplineVector;

	}else if(view == View::VM_TREEMAP_STRIP)
	{
		return TreeMapStripSplineVector;
	}
	else if(view == View::VM_RADIALTREE)
	{
		return RadialTreeSplineVector;
	}else{
		return NULL;
	}
}

bool lessThan(const BSpline *e1, const BSpline *e2 )
{
	if(e1->getLength() > e2->getLength()) return true;
	if(e1->getLength() == e2->getLength()) return true;

	return false;
}

void SplineSet::sortBSpline()
{
	qSort(ballonBSplineVector->begin(), ballonBSplineVector->end(), lessThan);
	qSort(rootedBSplineVector->begin(), rootedBSplineVector->end(), lessThan);
	qSort(treeMapSDBSplineVector->begin(), treeMapSDBSplineVector->end(), lessThan);
	qSort(TreeMapStripSplineVector->begin(), TreeMapStripSplineVector->end(), lessThan);
	qSort(RadialTreeSplineVector->begin(), RadialTreeSplineVector->end(), lessThan);
	
	for(int i = 0; i < ballonBSplineVector->size(); i++)		{ballonBSplineVector->at(i)->SetVectorPos((float)(i)/(float)(ballonBSplineVector->size()));				ballonBSplineVector->at(i)->UpdateIDs((float)(i)/(float)(ballonBSplineVector->size()));}
	for(int i = 0; i < rootedBSplineVector->size(); i++)		{rootedBSplineVector->at(i)->SetVectorPos((float)(i)/(float)(rootedBSplineVector->size()));				rootedBSplineVector->at(i)->UpdateIDs((float)(i)/(float)(rootedBSplineVector->size()));}
	for(int i = 0; i < treeMapSDBSplineVector->size(); i++)		{treeMapSDBSplineVector->at(i)->SetVectorPos((float)(i)/(float)(treeMapSDBSplineVector->size()));		treeMapSDBSplineVector->at(i)->UpdateIDs((float)(i)/(float)(treeMapSDBSplineVector->size()));}
	for(int i = 0; i < TreeMapStripSplineVector->size(); i++)	{TreeMapStripSplineVector->at(i)->SetVectorPos((float)(i)/(float)(TreeMapStripSplineVector->size()));	TreeMapStripSplineVector->at(i)->UpdateIDs((float)(i)/(float)(TreeMapStripSplineVector->size()));}
	for(int i = 0; i < RadialTreeSplineVector->size(); i++)		{RadialTreeSplineVector->at(i)->SetVectorPos((float)(i)/(float)(RadialTreeSplineVector->size()));		RadialTreeSplineVector->at(i)->UpdateIDs((float)(i)/(float)(RadialTreeSplineVector->size()));}

}



